import os

from helper import run


def test_encrypt_multi_single_cert_pem(tmp_path):
    out, err, exitcode = run(
        "rauc encrypt "
        "--to openssl-enc/keys/rsa-4096/cert-000.pem "
        "--to openssl-enc/keys/rsa-4096/cert-001.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"good-crypt-bundle-unencrypted.raucb {tmp_path}/encrypted.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/encrypted.raucb")


def test_encrypt_single_multi_cert_pem(tmp_path):
    out, err, exitcode = run(
        "rauc encrypt "
        "--to openssl-enc/keys/rsa-4096/certs.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"good-crypt-bundle-unencrypted.raucb {tmp_path}/encrypted.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/encrypted.raucb")


def test_encrypt_single_multi_cert_pem_rsa_ecc_mixed(tmp_path):
    out, err, exitcode = run(
        "rauc encrypt "
        "--to openssl-enc/keys/rsa-4096/certs.pem "
        "--to openssl-enc/keys/ecc/certs.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"good-crypt-bundle-unencrypted.raucb {tmp_path}/encrypted.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/encrypted.raucb")


def test_encrypt_broken_multi_cert_pem(tmp_path):
    with open("openssl-enc/keys/rsa-4096/certs.pem") as infile:
        with open(f"{tmp_path}/certs.pem", "a") as outfile:
            outfile.writelines(infile.readlines()[:-5])

    out, err, exitcode = run(
        "rauc encrypt "
        f"--to {tmp_path}/certs.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"good-crypt-bundle-unencrypted.raucb {tmp_path}/encrypted.raucb"
    )

    assert exitcode == 1

    assert not os.path.exists(f"{tmp_path}/encrypted.raucb")


def test_encrypt_missing_cert_in_file(tmp_path):
    out, err, exitcode = run(
        "rauc encrypt "
        "--to openssl-enc/keys/rsa-4096/private-key-001.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"good-crypt-bundle-unencrypted.raucb {tmp_path}/encrypted.raucb"
    )

    assert exitcode == 1
    assert "Expecting: CERTIFICATE" in err

    assert not os.path.exists(f"{tmp_path}/encrypted.raucb")


def test_encrypt_verity_bundle(tmp_path):
    out, err, exitcode = run(
        "rauc encrypt "
        "--to openssl-enc/keys/rsa-4096/cert-000.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"good-verity-bundle.raucb {tmp_path}/encrypted.raucb"
    )

    assert exitcode == 1
    assert "Refused to encrypt input bundle" in err

    assert not os.path.exists(f"{tmp_path}/encrypted.raucb")



#!/usr/bin/python3

import argparse
import json
import os
import socket
import sys

try:
    from aiohttp import web
except ModuleNotFoundError:
    print("aiohttp python module not installed, disabling RAUC_TEST_HTTP_BACKEND")
    exit(1)

routes = web.RouteTableDef()


def daemonize():
    if os.fork():
        sys.exit()

    os.setsid()

    if os.fork():
        sys.exit()


def open_socket(name):
    try:
        os.unlink(name)
    except FileNotFoundError:
        pass
    s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    s.bind(name)
    os.chmod(name, 0o666)
    return s


@routes.get("/")
async def index(request):
    return web.Response(text="OK")


@routes.get("/sporadic.raucb")
async def sporadic_get(request):
    request.app["rauc"]["sporadic_counter"] += 1
    if request.app["rauc"]["sporadic_counter"] % 2 == 0:
        raise web.HTTPInternalServerError()
    else:
        return web.FileResponse(path="test/good-verity-bundle.raucb")


@routes.get("/token.raucb")
async def token_get(request):
    if "token" not in request.cookies:
        raise web.HTTPUnauthorized(text="missing cookie token")
    elif request.cookies["token"] != "secret":
        raise web.HTTPUnauthorized(text="bad cookie token")
    else:
        return web.FileResponse(path="test/good-verity-bundle.raucb")


def reset_summary(request):
    request.app["rauc"]["summary"] = {
        "first_request_headers": {},
        "second_request_headers": {},
        "requests": 0,
        "range_requests": [],
    }


@routes.post("/setup")
async def setup_handler(request):
    try:
        data = await request.json()
        if "file_path" not in data:
            return web.Response(text="missing file_path", status=400)

        request.app["rauc"]["config"] = data
        reset_summary(request)

        return web.Response(text="OK")
    except json.JSONDecodeError:
        return web.Response(text="bad JSON", status=400)


@routes.get("/get")
async def get_handler(request):
    config = request.app["rauc"].get("config", {})
    if not config:
        return web.Response(text="not set up yet", status=400)

    summary = request.app["rauc"]["summary"]
    summary["requests"] += 1

    if not summary["first_request_headers"]:
        summary["first_request_headers"] = dict(request.headers)
    elif not summary["second_request_headers"]:
        summary["second_request_headers"] = dict(request.headers)

    if request.http_range:
        start = str(request.http_range.start) or ""
        end = str(request.http_range.stop) or ""
        summary["range_requests"].append(f"{start}:{end}")

    return web.FileResponse(config["file_path"])


@routes.get("/summary")
async def summary_handler(request):
    summary = request.app["rauc"].get("summary", {})
    reset_summary(request)
    return web.json_response(summary)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run aiohttp server with optional socket and daemon mode.")
    parser.add_argument("-s", "--socket", help="Path to Unix domain socket")
    parser.add_argument("-d", "--daemon", action="store_true", help="Run as daemon")

    args = parser.parse_args()

    app_args = {}

    if args.socket:
        app_args["sock"] = open_socket(args.socket)
    else:
        app_args["host"] = "127.0.0.1"
        app_args["port"] = 8080

    if args.daemon:
        daemonize()

    app = web.Application()
    app["rauc"] = {
        "sporadic_counter": -1,
    }
    app.add_routes(routes)
    web.run_app(app, **app_args)



import json
import shutil

from conftest import have_json, no_service, root
from helper import run


@no_service
def test_status_no_service(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status")

    assert exitcode == 0
    assert "=== System Info ===" in out


@no_service
def test_status_no_service_output_readable(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status --output-format=readable")

    assert exitcode == 0
    assert "=== System Info ===" in out


@no_service
def test_status_no_service_output_shell(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status --output-format=shell")

    assert exitcode == 0
    assert "RAUC_SYSTEM_COMPATIBLE='Test Config'" in out


@no_service
@have_json
def test_status_no_service_output_json(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status --output-format=json")

    assert exitcode == 0
    assert '"compatible":"Test Config"' in out


@no_service
@have_json
def test_status_no_service_output_json_pretty(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status --output-format=json-pretty")

    assert exitcode == 0
    assert '"compatible" : "Test Config"' in out


@no_service
@have_json
def test_status_no_service_output_invalid(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status --output-format=invalid")

    assert exitcode == 1
    assert "Unknown output format: 'invalid'" in err


def test_status(rauc_dbus_service_with_system):
    out, err, exitcode = run("rauc status")

    assert exitcode == 0
    assert out.startswith("=== System Info ===")


def test_status_readable(rauc_dbus_service_with_system):
    out, err, exitcode = run("rauc status --detailed --output-format=readable")

    assert exitcode == 0
    assert out.startswith("=== System Info ===")


def test_status_shell(rauc_dbus_service_with_system):
    out, err, exitcode = run("rauc status --detailed --output-format=shell")

    assert exitcode == 0
    assert out.startswith("RAUC_SYSTEM_COMPATIBLE='Test Config'")


@have_json
def test_status_json(rauc_dbus_service_with_system):
    out, err, exitcode = run("rauc status --detailed --output-format=json")

    assert exitcode == 0
    assert json.loads(out)


@have_json
def test_status_json_pretty(rauc_dbus_service_with_system):
    out, err, exitcode = run("rauc status --detailed --output-format=json-pretty")

    assert exitcode == 0
    assert json.loads(out)


def test_status_invalid(rauc_dbus_service_with_system):
    out, err, exitcode = run("rauc status --detailed --output-format=invalid")

    assert exitcode == 1
    assert "Unknown output format" in err


@root
def test_status_after_install(rauc_dbus_service_with_system, tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-verity-bundle.raucb", tmp_path / "good-verity-bundle.raucb")

    out, err, exitcode = run(f"rauc install {tmp_path}/good-verity-bundle.raucb")
    assert exitcode == 0

    out, err, exitcode = run("rauc status --detailed --output-format=readable")
    assert exitcode == 0

    out, err, exitcode = run("rauc status --detailed --output-format=json")
    assert exitcode == 0
    assert json.loads(out)

    out, err, exitcode = run("rauc status --detailed --output-format=shell")
    assert exitcode == 0



# -*- coding: utf-8 -*-
#
# rauc documentation build configuration file, created by
# sphinx-quickstart on Fri Jan 22 16:00:15 2016.

import sys
import os
import subprocess

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#sys.path.insert(0, os.path.abspath('.'))

# -- General configuration ------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.extlinks',
    'sphinx.ext.ifconfig',
    'sphinx_rtd_theme',
]

try:
    import sphinxext.opengraph
    extensions.append('sphinxext.opengraph')
    ogp_site_url = 'https://rauc.readthedocs.io/en/latest/'
    ogp_image = 'https://rauc.readthedocs.io/en/latest/_static/RAUC_Logo_outline.svg'
except ModuleNotFoundError:
    print("not using sphinxext.opengraph")

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'RAUC'
author = u'Jan Luebbe, Enrico Joerns, the RAUC contributors'
copyright = u'2016-2026, ' + author

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = subprocess.check_output(['../build-aux/git-version-gen', '../.tarball-version']).decode()
if version.endswith('-dirty'):
  version = version[:-6]
# The full version, including alpha/beta/rc tags.
release = version

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'sphinx_rtd_theme'

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = 'RAUC_Logo_outline.svg'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = 'favicon.ico'

# -- Options for LaTeX output ---------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
  ('index', 'rauc.tex', u'RAUC Documentation',
   author, 'manual'),
]

# -- Options for manual page output ---------------------------------------

man_pages = [
  ('man', 'rauc', 'safe and secure updating', '', 1),
]

# -- Options for external links -------------------------------------------

extlinks = {
  'pr': ('https://github.com/rauc/rauc/pull/%s', '#%s'),
}



import json
import shutil
import subprocess

from conftest import have_json, have_streaming
from helper import run


def test_info_plain(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info {tmp_path}/good-bundle.raucb")

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "Bundle Format:  plain" in out


def test_info_verity():
    out, err, exitcode = run("rauc --keyring openssl-ca/dev-ca.pem info good-verity-bundle.raucb")

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "Bundle Format:  verity" in out


def test_info_verity_adaptive_meta():
    out, err, exitcode = run("rauc --keyring openssl-ca/dev-ca.pem info good-adaptive-meta-bundle.raucb")

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")


@have_streaming
def test_info_streaming():
    out, err, exitcode = run(
        "rauc --keyring openssl-ca/dev-ca.pem info http://127.0.0.1/test/good-verity-bundle.raucb"
    )

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")


@have_streaming
def test_info_streaming_key_file():
    out, err, exitcode = run(
        "rauc --keyring openssl-ca/dev-ca.pem "
        "info https://127.0.0.3/test/good-verity-bundle.raucb "
        "--tls-cert openssl-ca/web/client-1.cert.pem "
        "--tls-key openssl-ca/web/private/client-1.pem "
        "--tls-ca openssl-ca/web/ca.cert.pem"
    )

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")


@have_streaming
def test_info_streaming_key_pkcs11(pkcs11):
    out, err, exitcode = run(
        "rauc --keyring openssl-ca/dev-ca.pem "
        "-C streaming:sandbox-user=root "  # softhsm DB is currently in /tmp/pytest-of-root, which is root-only
        "info https://127.0.0.3/test/good-verity-bundle.raucb "
        "--tls-cert openssl-ca/web/client-1.cert.pem "
        "--tls-key 'pkcs11:token=rauc;object=client-1?pin-value=1111' "
        "--tls-ca openssl-ca/web/ca.cert.pem"
    )

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")


def test_info_casync_plain(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-casync-bundle-1.5.1.raucb", tmp_path / "good-casync-bundle-1.5.1.raucb")

    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info {tmp_path}/good-casync-bundle-1.5.1.raucb")

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "rootfs.img.caibx" in out
    assert "appfs.img.caibx" in out


def test_info_casync_verity():
    out, err, exitcode = run("rauc --keyring openssl-ca/dev-ca.pem info good-casync-bundle-verity.raucb")

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "rootfs.img.caibx" in out
    assert "appfs.img.caibx" in out


def test_info_crypt_unencrypted():
    out, err, exitcode = run("rauc --keyring openssl-ca/dev-ca.pem info good-crypt-bundle-unencrypted.raucb")

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "Bundle Format:  crypt" in out


def test_info_crypt_encrypted_valid_key():
    out, err, exitcode = run(
        "rauc --keyring openssl-ca/dev-ca.pem "
        "--key openssl-enc/keys/rsa-4096/private-key-000.pem "
        "info good-crypt-bundle-encrypted.raucb"
    )

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "Bundle Format:  crypt" in out


def test_info_crypt_encrypted_invalid_key():
    out, err, exitcode = run(
        "rauc --keyring openssl-ca/dev-ca.pem "
        "--key openssl-enc/keys/rsa-4096/private-key-005.pem "
        "info good-crypt-bundle-encrypted.raucb"
    )

    assert exitcode == 1
    assert "Failed to decrypt CMS EnvelopedData" in err


def test_info_crypt_encrypted_pkcs11(pkcs11):
    out, err, exitcode = run(
        "rauc --keyring openssl-ca/dev-ca.pem "
        "--key 'pkcs11:token=rauc;object=enc-rsa-000' "
        "info good-crypt-bundle-encrypted.raucb"
    )

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "Bundle Format:  crypt" in out


def test_info_dump_recipients_crypt_encrypted():
    out, err, exitcode = run(
        "rauc --keyring openssl-ca/dev-ca.pem "
        "--key openssl-enc/keys/rsa-4096/private-key-000.pem "
        "--dump-recipients "
        "info good-crypt-bundle-encrypted.raucb"
    )

    assert exitcode == 0
    assert "10 Recipients:" in out


def test_info_with_config(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(f"rauc --conf=test.conf info {tmp_path}/good-bundle.raucb")

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "Bundle Format:  plain" in out


def test_info_verification_failure(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("invalid-sig-bundle.raucb", tmp_path / "invalid-sig-bundle.raucb")

    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info {tmp_path}/invalid-sig-bundle.raucb")

    assert exitcode == 1
    assert "signature verification failed" in err


def test_info_dump_cert_unverified():
    out, err, exitcode = run("rauc --no-verify --dump-cert info good-bundle.raucb")

    assert exitcode == 0
    assert "Certificate:" in out


def test_info_valid_file_uri(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(f"rauc info --keyring openssl-ca/dev-ca.pem file://{tmp_path}/good-bundle.raucb")

    assert exitcode == 0
    assert out.startswith("Compatible:     'Test Config'")
    assert "Bundle Format:  plain" in out


def test_info_invalid_file_uri():
    out, err, exitcode = run("rauc info --keyring openssl-ca/dev-ca.pem file://good-bundle.raucb")

    assert exitcode == 1
    assert "Conversion error:" in err

    out, err, exitcode = run("rauc info --keyring openssl-ca/dev-ca.pem file:/good-bundle.raucb")

    assert exitcode == 1
    assert "No such file:" in err


def test_info_format_shell(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-adaptive-meta-bundle.raucb", tmp_path / "good-adaptive-meta-bundle.raucb")

    proc = subprocess.run(
        "rauc info --keyring openssl-ca/dev-ca.pem "
        f"--output-format=shell {tmp_path}/good-adaptive-meta-bundle.raucb | sh",
        shell=True,
    )
    assert proc.returncode == 0


@have_json
def test_info_format_json(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        f"rauc info --keyring openssl-ca/dev-ca.pem --output-format=json {tmp_path}/good-bundle.raucb"
    )

    assert exitcode == 0
    assert json.loads(out)


@have_json
def test_info_format_json_pretty(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        f"rauc info --keyring openssl-ca/dev-ca.pem --output-format=json-pretty {tmp_path}/good-bundle.raucb"
    )

    assert exitcode == 0
    assert json.loads(out)


@have_json
def test_info_format_json2(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        f"rauc info --keyring openssl-ca/dev-ca.pem --output-format=json-2 {tmp_path}/good-bundle.raucb"
    )

    assert exitcode == 0
    assert json.loads(out)


def test_info_format_invalid():
    out, err, exitcode = run("rauc info --keyring openssl-ca/dev-ca.pem --output-format=invalid good-bundle.raucb")

    assert exitcode == 1
    assert "Unknown output format: 'invalid'" in err


def test_info_configoverride(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(f"rauc info -c test.conf -C system:bundle-formats=verity {tmp_path}/good-bundle.raucb")
    assert exitcode == 1
    assert "Bundle format 'plain' not allowed" in err

    out, err, exitcode = run("rauc info -C system:bundle-formats")
    assert exitcode == 1
    assert "Error in parsing override: Missing '='" in err

    out, err, exitcode = run("rauc info -C systembundle-formats=verity")
    assert exitcode == 1
    assert "Error in parsing override: Missing ':'" in err

    out, err, exitcode = run("rauc info -C")
    assert exitcode == 1
    assert "Error in parsing override: you must specify it in format SECTION:KEY=VALUE" in err



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2021 Enrico Jörns <e.joerns@pengutronix.de>, Pengutronix
# SPDX-FileCopyrightText: 2021-2022 Bastian Krause <bst@pengutronix.de>, Pengutronix

import logging
import os
import shlex
import subprocess


def logger_from_command(command):
    """
    Returns a logger named after the executable, or in case of a python executable, after the
    python module,
    """
    cmd_parts = command.split()
    base_cmd = os.path.basename(cmd_parts[0])
    try:
        if base_cmd.startswith("python") and cmd_parts[1] == "-m":
            base_cmd = command.split()[2]
    except IndexError:
        pass

    return logging.getLogger(base_cmd)


def run(command, *, timeout=30):
    """
    Runs given command as subprocess with DBUS_STARTER_BUS_TYPE=session and PATH+=./build. Blocks
    until command terminates. Logs command and its stdout/stderr/exit code (use
    --log-cli-level=info to show them with pytest).
    Returns tuple (stdout, stderr, exit code).
    """
    logger = logger_from_command(command)
    logger.info("running: %s", command)

    proc = subprocess.run(shlex.split(command), capture_output=True, text=True, check=False, timeout=timeout)

    for line in proc.stdout.splitlines():
        if line:
            logger.info("stdout: %s", line)
    for line in proc.stderr.splitlines():
        if line:
            logger.warning("stderr: %s", line)

    logger.info("exitcode: %d", proc.returncode)

    return proc.stdout, proc.stderr, proc.returncode


def run_tree(path):
    subprocess.check_call(["tree", "--metafirst", "-ax", "-pugs", "--inodes", path])


def slot_data_from_json(status_data, slotname):
    """
    Helper to return the slot data from 'rauc status' JSON output.

    :param status_data: JSON data obtained from 'rauc status'.
    :param slotname: The name of the slot to find in the data.
    :return: Slot data as a dictionary.
    """
    for slot in status_data["slots"]:
        if slotname in slot:
            return slot[slotname]
    else:
        raise ValueError(f"Slot '{slotname}' not found")



import os
import shutil

from conftest import have_casync, have_desync
from helper import run


@have_casync
def test_convert(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        f" convert {tmp_path}/good-bundle.raucb {tmp_path}/casync.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/casync.raucb")


@have_casync
def test_convert_ignore_image(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        " convert"
        " --ignore-image appfs"
        f" {tmp_path}/good-bundle.raucb {tmp_path}/casync.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/casync.raucb")


@have_casync
def test_convert_output_exists(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    open(f"{tmp_path}/casync.raucb", "a").close()

    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        f" convert {tmp_path}/good-bundle.raucb {tmp_path}/casync.raucb"
    )

    assert exitcode == 1
    assert "already exists" in err

    assert os.path.exists(f"{tmp_path}/casync.raucb")


@have_casync
def test_convert_error(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/rel/release-2018.cert.pem"
        " --key openssl-ca/rel/private/release-2018.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" convert {tmp_path}/good-bundle.raucb {tmp_path}/casync.raucb"
    )

    assert exitcode == 1

    assert not os.path.exists(f"{tmp_path}/casync.raucb")


@have_casync
def test_convert_casync_extra_args(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        " convert"
        ' --casync-args="--chunk-size=64000"'
        f" {tmp_path}/good-bundle.raucb {tmp_path}/casync-extra-args.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/casync-extra-args.raucb")
    assert os.path.isdir(f"{tmp_path}/casync-extra-args.castr")


@have_casync
def test_convert_verity(tmp_path):
    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        f" bundle install-content/ {tmp_path}/tmp-verity.raucb"
    )
    assert exitcode == 0

    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        " --trust-environment"
        f" convert {tmp_path}/tmp-verity.raucb {tmp_path}/casync-verity.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/casync-verity.raucb")
    assert os.path.isdir(f"{tmp_path}/casync-verity.castr")


@have_desync
def test_convert_desync(tmp_path, system):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    system.config["casync"] = {
        "use-desync": "true",
    }
    system.write_config()

    out, err, exitcode = run(
        f"{system.prefix}"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        f" convert {tmp_path}/good-bundle.raucb {tmp_path}/desync.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/desync.raucb")
    assert os.path.isdir(f"{tmp_path}/desync.castr")


@have_desync
def test_convert_desync_output_exists(tmp_path, system):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    open(f"{tmp_path}/desync.raucb", "a").close()

    system.config["casync"] = {
        "use-desync": "true",
    }
    system.write_config()

    out, err, exitcode = run(
        f"{system.prefix}"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        f" convert {tmp_path}/good-bundle.raucb {tmp_path}/desync.raucb"
    )

    assert exitcode == 1
    assert "already exists" in err

    assert os.path.exists(f"{tmp_path}/desync.raucb")


@have_desync
def test_convert_desync_error(tmp_path, system):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    system.config["casync"] = {
        "use-desync": "true",
    }
    system.write_config()

    out, err, exitcode = run(
        f"{system.prefix}"
        " --cert openssl-ca/rel/release-2018.cert.pem"
        " --key openssl-ca/rel/private/release-2018.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" convert {tmp_path}/good-bundle.raucb {tmp_path}/desync.raucb"
    )

    assert exitcode == 1

    assert not os.path.exists(f"{tmp_path}/desync.raucb")


@have_desync
def test_convert_desync_extra_args(tmp_path, system):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    system.config["casync"] = {
        "use-desync": "true",
    }
    system.write_config()

    out, err, exitcode = run(
        f"{system.prefix}"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        " convert"
        ' --casync-args="--chunk-size=32:128:512"'
        f" {tmp_path}/good-bundle.raucb {tmp_path}/desync-extra-args.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/desync-extra-args.raucb")



import os

from conftest import have_faketime
from helper import run


@have_faketime
def test_sign_bundle_expired_cert(tmp_path):
    out, err, exitcode = run(
        ' faketime "2019-07-02" '
        " rauc "
        "--cert openssl-ca/rel/release-2018.cert.pem "
        "--key openssl-ca/rel/private/release-2018.pem "
        "--keyring openssl-ca/rel-ca.pem "
        f"bundle install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 1
    assert "certificate has expired" in err

    assert not os.path.exists(f"{tmp_path}/out.raucb")


@have_faketime
def test_sign_bundle_not_yet_valid_cert(tmp_path):
    out, err, exitcode = run(
        ' faketime "2017-01-01" '
        " rauc "
        "--cert openssl-ca/rel/release-2018.cert.pem "
        "--key openssl-ca/rel/private/release-2018.pem "
        "--keyring openssl-ca/rel-ca.pem "
        f"bundle install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 1
    assert "certificate is not yet valid" in err

    assert not os.path.exists(f"{tmp_path}/out.raucb")


@have_faketime
def test_sign_bundle_almost_expired_cert(tmp_path):
    out, err, exitcode = run(
        ' faketime "2019-06-15" '
        " rauc "
        "--cert openssl-ca/rel/release-2018.cert.pem "
        "--key openssl-ca/rel/private/release-2018.pem "
        "--keyring openssl-ca/rel-ca.pem "
        f"bundle install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/out.raucb")


@have_faketime
def test_sign_bundle_valid_cert(tmp_path):
    out, err, exitcode = run(
        ' faketime "2019-01-01" '
        " rauc "
        "--cert openssl-ca/rel/release-2018.cert.pem "
        "--key openssl-ca/rel/private/release-2018.pem "
        "--keyring openssl-ca/rel-ca.pem "
        f"bundle install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/out.raucb")


@have_faketime
def test_sign_bundle_valid_cert_encrypted_key(tmp_path, monkeypatch):
    monkeypatch.setenv("RAUC_KEY_PASSPHRASE", "1111")
    out, err, exitcode = run(
        'faketime "2019-01-01" '
        "rauc "
        "--cert openssl-ca/rel/release-1.cert.pem "
        "--key openssl-ca/rel/private/release-1-encrypted.pem "
        "--keyring openssl-ca/rel-ca.pem "
        f"bundle install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/out.raucb")



import os
import shutil

from conftest import have_faketime
from helper import run

# all tests require faketime to be available
pytestmark = have_faketime


def prepare_signing_time_conf(tmp_path):
    shutil.copyfile("openssl-ca/dev-ca.pem", tmp_path / "test-ca.pem")
    with open(f"{tmp_path}/use-bundle-signing-time.conf", "a") as f:
        f.write("""
[system]
compatible=Test Config
bootloader=grub
grubenv=grubenv.test

[keyring]
path=test-ca.pem
use-bundle-signing-time=true
""")


def test_verify_with_use_bundle_signing_time_valid_signing_invalid_current(tmp_path):
    shutil.copytree("install-content", tmp_path / "install-content")
    prepare_signing_time_conf(tmp_path)

    out, err, exitcode = run(
        'faketime "2018-01-01" '
        "rauc "
        "--cert openssl-ca/rel/release-2018.cert.pem "
        "--key openssl-ca/rel/private/release-2018.pem "
        "--keyring openssl-ca/rel-ca.pem "
        f" bundle {tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(
        f'faketime "2022-01-01" rauc --conf {tmp_path}/use-bundle-signing-time.conf info {tmp_path}/out.raucb'
    )

    assert exitcode == 0


def test_verify_with_use_bundle_signing_time_invalid_signing_valid_current(tmp_path):
    shutil.copytree("install-content", tmp_path / "install-content")
    prepare_signing_time_conf(tmp_path)

    out, err, exitcode = run(
        'faketime "2022-01-01" '
        "rauc "
        "--cert openssl-ca/rel/release-2018.cert.pem "
        "--key openssl-ca/rel/private/release-2018.pem "
        f" bundle {tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(
        f'faketime "2018-01-01" rauc --conf {tmp_path}/use-bundle-signing-time.conf info {tmp_path}/out.raucb'
    )

    assert exitcode == 1


def test_info_no_check_time(tmp_path):
    shutil.copytree("install-content", tmp_path / "install-content")

    out, err, exitcode = run(
        'faketime "2018-01-01" '
        "rauc "
        "--cert openssl-ca/rel/release-2018.cert.pem "
        "--key openssl-ca/rel/private/release-2018.pem "
        f" bundle {tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(f'faketime "2018-01-01" rauc --keyring openssl-ca/rel-ca.pem info {tmp_path}/out.raucb')

    assert exitcode == 0

    out, err, exitcode = run(f'faketime "2022-01-01" rauc --keyring openssl-ca/rel-ca.pem info {tmp_path}/out.raucb')

    assert exitcode == 1

    out, err, exitcode = run(
        f'faketime "2022-01-01" rauc --keyring openssl-ca/rel-ca.pem --no-check-time info {tmp_path}/out.raucb'
    )

    assert exitcode == 0



import os
import shutil

from helper import run


def test_bundle(tmp_path):
    shutil.copytree("install-content", tmp_path / "install-content")

    out, err, exitcode = run(
        "rauc bundle "
        "--cert openssl-ca/dev/autobuilder-1.cert.pem "
        "--key openssl-ca/dev/private/autobuilder-1.pem "
        f"{tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0
    assert "Creating 'verity' format bundle" in out

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(f"rauc -c test.conf info {tmp_path}/out.raucb")
    assert exitcode == 0


def test_bundle_args_compat(tmp_path):
    "test compatibility for cert/key args before subcommand"

    shutil.copytree("install-content", tmp_path / "install-content")

    out, err, exitcode = run(
        f"rauc \
            --cert openssl-ca/dev/autobuilder-1.cert.pem \
            --key openssl-ca/dev/private/autobuilder-1.pem \
            bundle \
            {tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0
    assert "Creating 'verity' format bundle" in out

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(f"rauc -c test.conf info {tmp_path}/out.raucb")
    assert exitcode == 0


def test_bundle_mksquashfs_extra_args(tmp_path):
    shutil.copytree("install-content", tmp_path / "install-content")

    out, err, exitcode = run(
        f'rauc \
            --cert openssl-ca/dev/autobuilder-1.cert.pem \
            --key openssl-ca/dev/private/autobuilder-1.pem \
            bundle \
            --mksquashfs-args="-comp xz -info" \
            {tmp_path}/install-content {tmp_path}/out.raucb'
    )

    assert exitcode == 0
    assert "Creating 'verity' format bundle" in out

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(f"rauc -c test.conf info {tmp_path}/out.raucb")
    assert exitcode == 0


def test_bundle_pkcs11_key1(tmp_path, pkcs11):
    "A bundle signed with autobuilder-1 key must verify against keyring"

    shutil.copytree("install-content", tmp_path / "install-content")

    out, err, exitcode = run(
        f"rauc bundle \
            --cert 'pkcs11:token=rauc;object=autobuilder-1' \
            --key 'pkcs11:token=rauc;object=autobuilder-1' \
            {tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0
    assert "Creating 'verity' format bundle" in out

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(f"rauc -c test.conf info {tmp_path}/out.raucb")
    assert exitcode == 0


def test_bundle_pkcs11_key2_revoked(tmp_path, pkcs11):
    "A bundle signed with revoked autobuilder-2 key must NOT verify against keyring"

    shutil.copytree("install-content", tmp_path / "install-content")

    out, err, exitcode = run(
        f"rauc bundle \
            --cert 'pkcs11:token=rauc;object=autobuilder-2' \
            --key 'pkcs11:token=rauc;object=autobuilder-2' \
            {tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0
    assert "Creating 'verity' format bundle" in out

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(f"rauc -c test.conf info {tmp_path}/out.raucb")
    assert exitcode == 1
    assert "certificate revoked" in err


def test_bundle_pkcs11_key_mismatch(tmp_path, pkcs11):
    "A bundle cannot be signed with mismatching key pair"

    shutil.copytree("install-content", tmp_path / "install-content")

    out, err, exitcode = run(
        f"rauc bundle \
            --cert 'pkcs11:token=rauc;object=autobuilder-1' \
            --key 'pkcs11:token=rauc;object=autobuilder-2' \
            {tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 1
    assert "Creating 'verity' format bundle" in out
    assert "key values mismatch" in err

    assert not os.path.exists(f"{tmp_path}/out.raucb")


def test_bundle_crypt(tmp_path):
    shutil.copytree("install-content", tmp_path / "install-content")
    shutil.copyfile("install-content/manifest.raucm.crypt", tmp_path / "install-content/manifest.raucm")

    out, err, exitcode = run(
        "rauc bundle "
        "--cert openssl-ca/dev/autobuilder-1.cert.pem "
        "--key openssl-ca/dev/private/autobuilder-1.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"{tmp_path}/install-content {tmp_path}/out.raucb"
    )

    assert exitcode == 0
    assert "Creating 'crypt' format bundle" in out

    assert os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(f"rauc -c test.conf info {tmp_path}/out.raucb")
    assert exitcode == 0


def test_bundle_adaptive(tmp_path, bundle):
    bundle.manifest["image.rootfs"] = {
        "filename": "rootfs.img",
        "adaptive": "block-hash-index",
    }
    bundle.make_random_image("rootfs", 4097, "random rootfs")
    out, err, exitcode = bundle.build_nocheck()
    assert exitcode == 1
    assert "Creating 'verity' format bundle" in out
    assert "not a multiple of 4096 bytes" in err
    assert not bundle.output.is_file()


def test_bundle_content_checks(tmp_path, bundle):
    bundle.build()
    assert bundle.output.exists()
    bundle.output.unlink()

    # absolute symlinks are not allowed
    test_file = bundle.content / "abs-symlink"
    test_file.symlink_to("/dev/null")
    out, err, exitcode = bundle.build_nocheck()
    assert exitcode == 1
    assert "absolute symlinks are not supported as bundle contents (abs-symlink)" in err
    assert not bundle.output.is_file()
    test_file.unlink()

    # relative symlinks are not allowed
    test_file = bundle.content / "rel-symlink"
    test_file.symlink_to("../foo")
    out, err, exitcode = bundle.build_nocheck()
    assert exitcode == 1
    assert "symlinks containing slashes are not supported as bundle contents (rel-symlink)" in err
    assert not bundle.output.is_file()
    test_file.unlink()

    # local symlinks are allowed
    test_file = bundle.content / "local-symlink"
    test_file.symlink_to("foo")
    bundle.build()
    assert bundle.output.is_file()
    bundle.output.unlink()
    test_file.unlink()

    # non-directories are allowed
    test_dir = bundle.content / "subdir"
    test_dir.mkdir()
    bundle.build()
    assert bundle.output.is_file()
    bundle.output.unlink()
    test_dir.rmdir()

    # hidden directories are not allowed
    test_dir = bundle.content / ".hidden_subdir"
    test_dir.mkdir()
    out, err, exitcode = bundle.build_nocheck()
    assert exitcode == 1
    assert "hidden directories are not supported as bundle contents (.hidden_subdir)"
    assert not bundle.output.is_file()
    test_dir.rmdir()

    # fifos are not allowed
    test_fifo = bundle.content / "fifo"
    os.mkfifo(test_fifo)
    out, err, exitcode = bundle.build_nocheck()
    assert exitcode == 1
    assert "only regular files are supported as bundle contents (fifo)"
    assert not bundle.output.is_file()
    test_fifo.unlink()


def test_bundle_min_rauc_version(bundle):
    bundle.manifest["update"]["min-rauc-version"] = "1.14-dev"
    bundle.build()
    bundle.output.unlink()

    bundle.manifest["update"]["min-rauc-version"] = "1000-rc.1+21000101"
    out, err, exitcode = bundle.build_nocheck()
    assert exitcode == 1
    assert "Creating 'verity' format bundle" in out
    assert "Minimum RAUC version in manifest (1000-rc.1+21000101) is newer than current version" in err
    assert not bundle.output.is_file()

    bundle.manifest["update"]["min-rauc-version"] = "bad_version"
    out, err, exitcode = bundle.build_nocheck()
    assert exitcode == 1
    assert "Creating 'verity' format bundle" in out
    assert (
        "Failed to parse 'min-rauc-version'. Expected 'Major[.Minor[.Patch]][-pre_release]]', got 'bad_version'" in err
    )
    assert not bundle.output.is_file()

    bundle.manifest["update"]["min-rauc-version"] = "1.13"
    out, err, exitcode = bundle.build_nocheck()
    assert exitcode == 1
    assert "Creating 'verity' format bundle" in out
    assert "Minimum RAUC version field in manifest is only supported since 1.14 (not '1.13')" in err
    assert not bundle.output.is_file()


def test_rollout_options(bundle):
    bundle.manifest["rollout"] = {
        "foo": "bar",
    }
    bundle.build()



import shutil
import tarfile
import json
from io import BytesIO
from pprint import pprint
from pathlib import Path
from contextlib import contextmanager

from conftest import root, have_json, needs_composefs, Bundle
from helper import run, run_tree


pytestmark = [root, have_json]


def make_tarfile(path, contents):
    with tarfile.open(name=path, mode="w") as t:
        for filename, content in contents.items():
            f = tarfile.TarInfo(name=filename)
            c = BytesIO(content)
            f.size = len(content)
            t.addfile(f, c)


def get_info(path):
    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info --output-format=json-2 {path}")
    assert exitcode == 0

    info = json.loads(out)

    assert info
    assert info.get("bundle", {}).get("format") == "verity"

    return info


def get_composefs_info(image):
    out, err, exitcode = run(f"composefs-info ls {image}")
    assert exitcode == 0
    assert err == ""

    result = {}
    for line in out.splitlines():
        name, obj = line.split("@", 1)
        result[name.rstrip("\t")] = obj.strip()

    return result


class RepoStatus(dict):
    @property
    def artifacts(self):
        result = {}

        for artifact in self.get("artifacts", []):
            for instance in artifact["instances"]:
                result.setdefault(artifact["name"], {})[instance["checksum"]] = instance

        return result

    @property
    def referenced_artifacts(self):
        result = {}

        for artifact in self.get("artifacts", []):
            for instance in artifact["instances"]:
                if len(instance["references"]) == 0:
                    continue

                result.setdefault(artifact["name"], {})[instance["checksum"]] = instance

        return result

    @property
    def path(self):
        return Path(self["path"])


class Status(dict):
    @property
    def repos(self):
        result = {}

        for repo in self.get("artifact-repositories", []):
            result[repo["name"]] = RepoStatus(repo)

        return result


def get_status():
    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0

    status = json.loads(out)

    assert status
    assert status.get("compatible") == "Test Config"

    return Status(status)


@contextmanager
def extracted_bundle(tmp_path, bundle_path, remove=True):
    path = tmp_path / "extracted"
    assert not path.exists()

    try:
        out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem extract {bundle_path} {path}")
        assert exitcode == 0
        yield path
    finally:
        if remove:
            shutil.rmtree(path)


@contextmanager
def mounted_composefs(tmp_path, image_path, store_path):
    mount_path = tmp_path / "mounted-composefs"
    mount_path.mkdir(exist_ok=True)

    run(f"mount.composefs -o basedir={store_path} {image_path} {mount_path}")
    try:
        yield mount_path
    finally:
        run(f"umount {mount_path}")


def test_bundle(tmp_path, bundle):
    """Create a bundle with artifacts and check the resulting information."""
    bundle.manifest["image.files/artifact-1"] = {
        "filename": "file-a.raw",
    }
    bundle.manifest["image.trees/artifact-1"] = {
        "filename": "tree-a.tar",
    }
    with open(bundle.content / "file-a.raw", "wb") as f:
        data_a = b"content-a"
        f.write(data_a)
    make_tarfile(bundle.content / "tree-a.tar", {"file": data_a})
    bundle.build()

    info = get_info(bundle.output)
    assert info["images"][0]["filename"] == "file-a.raw"
    assert info["images"][0]["slot-class"] == "files"
    assert info["images"][0]["artifact"] == "artifact-1"
    assert info["images"][1]["filename"] == "tree-a.tar"
    assert info["images"][1]["slot-class"] == "trees"
    assert info["images"][1]["artifact"] == "artifact-1"

    with extracted_bundle(tmp_path, bundle.output) as extracted:
        assert (extracted / "file-a.raw").is_file()
        assert (extracted / "tree-a.tar").is_file()


def test_convert_info(tmp_path, bundle):
    """Create a bundle by extracting a tar file."""
    bundle.manifest["image.trees/artifact-1"] = {
        "filename": "tree-a.tar",
        "convert": "tar-extract",
    }
    data_a = b"content-a"
    make_tarfile(bundle.content / "tree-a.tar", {"file": data_a})
    bundle.build()

    info = get_info(bundle.output)
    assert info["images"][0]["filename"] == "tree-a.tar"
    assert info["images"][0]["slot-class"] == "trees"
    assert info["images"][0]["artifact"] == "artifact-1"
    assert info["images"][0]["convert"] == ["tar-extract"]
    assert info["images"][0]["converted"] == ["tree-a.tar.extracted"]

    # test old json output
    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info --output-format=json {bundle.output}")
    assert exitcode == 0
    info = json.loads(out)
    assert info["images"][0]["trees"]["artifact"] == "artifact-1"
    assert info["images"][0]["trees"]["filename"] == "tree-a.tar"
    assert info["images"][0]["trees"]["convert"] == ["tar-extract"]
    assert info["images"][0]["trees"]["converted"] == ["tree-a.tar.extracted"]

    # test readable output
    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info --output-format=readable {bundle.output}")
    assert exitcode == 0
    out = out.splitlines()
    assert "    Convert:   tar-extract" in out
    assert "    Converted: 'tree-a.tar.extracted'" in out

    # test shell output
    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info --output-format=shell {bundle.output}")
    assert exitcode == 0
    assert out
    out = out.splitlines()
    assert "RAUC_IMAGE_CONVERT_0='tar-extract'" in out
    assert "RAUC_IMAGE_CONVERTED_0='tree-a.tar.extracted'" in out


def test_bundle_convert_tree(tmp_path, bundle):
    """Create a bundle by extracting a tar file."""
    bundle.manifest["image.trees/artifact-1"] = {
        "filename": "tree-a.tar",
        "convert": "tar-extract",
    }
    data_a = b"content-a"
    make_tarfile(bundle.content / "tree-a.tar", {"file": data_a})
    bundle.build()

    info = get_info(bundle.output)
    pprint(info)
    assert info["images"][0]["filename"] == "tree-a.tar"
    assert info["images"][0]["slot-class"] == "trees"
    assert info["images"][0]["artifact"] == "artifact-1"
    assert info["images"][0]["convert"] == ["tar-extract"]
    assert info["images"][0]["converted"] == ["tree-a.tar.extracted"]

    with extracted_bundle(tmp_path, bundle.output) as extracted:
        run_tree(extracted)
        assert not (extracted / "tree-a.tar").exists()
        assert (extracted / "tree-a.tar.extracted").is_dir()
        assert (extracted / "tree-a.tar.extracted/file").is_file()
        with open(extracted / "tree-a.tar.extracted/file", "rb") as f:
            assert f.read() == data_a


def test_bundle_convert_tree_keep(tmp_path, bundle):
    """Create a bundle by extracting a tar file while keeping the original."""
    bundle.manifest["image.trees/artifact-1"] = {
        "filename": "tree-a.tar",
        "convert": "tar-extract;keep",
    }
    make_tarfile(bundle.content / "tree-a.tar", {"file": b"contents-a"})
    bundle.build()

    info = get_info(bundle.output)
    pprint(info)
    assert info["images"][0]["filename"] == "tree-a.tar"
    assert info["images"][0]["slot-class"] == "trees"
    assert info["images"][0]["artifact"] == "artifact-1"
    assert info["images"][0]["convert"] == ["tar-extract", "keep"]
    assert info["images"][0]["converted"] == ["tree-a.tar.extracted", "tree-a.tar"]

    with extracted_bundle(tmp_path, bundle.output) as extracted:
        assert (extracted / "tree-a.tar").exists()
        assert (extracted / "tree-a.tar.extracted").is_dir()
        assert (extracted / "tree-a.tar.extracted/file").is_file()
        with open(extracted / "tree-a.tar.extracted/file", "rb") as f:
            assert f.read() == b"contents-a"


@needs_composefs
def test_bundle_convert_composefs(tmp_path, bundle):
    """Create a bundle by converting a tar file to a composfs image and objects."""
    bundle.manifest["image.trees/artifact-1"] = {
        "filename": "tree-a.tar",
        "convert": "composefs",
    }
    data_a = b"content-a" * 1024
    data_b = b"content-b" * 1024
    make_tarfile(
        bundle.content / "tree-a.tar",
        {
            "file-a-1": data_a,
            "file-a-2": data_a,
            "file-b": data_b,
        },
    )
    bundle.build()

    info = get_info(bundle.output)
    pprint(info)
    assert info["images"][0]["filename"] == "tree-a.tar"
    assert info["images"][0]["slot-class"] == "trees"
    assert info["images"][0]["artifact"] == "artifact-1"
    assert info["images"][0]["convert"] == ["composefs"]
    assert info["images"][0]["converted"] == ["tree-a.tar.cfs"]

    with extracted_bundle(tmp_path, bundle.output, remove=False) as extracted:
        run_tree(extracted)

        image_path = extracted / "tree-a.tar.cfs" / "image.cfs"
        store_path = extracted / ".rauc-cfs-store"

        assert not (extracted / "tree-a.tar").exists()
        assert image_path.is_file()
        assert store_path.is_dir()

        composefs_info = get_composefs_info(extracted / "tree-a.tar.cfs/image.cfs")
        assert set(composefs_info.keys()) == {
            "/file-a-1",
            "/file-a-2",
            "/file-b",
        }
        # both files should share one object
        assert composefs_info["/file-a-1"] == composefs_info["/file-a-2"]

        with mounted_composefs(tmp_path, image_path, store_path) as mount_path:
            with open(mount_path / "file-a-1", "rb") as f:
                assert f.read() == data_a
            with open(mount_path / "file-a-2", "rb") as f:
                assert f.read() == data_a
            with open(mount_path / "file-b", "rb") as f:
                assert f.read() == data_b


def do_install_file(tmp_path, name, repo_name, artifact_name, artifact_data):
    bundle = Bundle(tmp_path, name)
    bundle.manifest[f"image.{repo_name}/{artifact_name}"] = {
        "filename": "file.raw",
    }
    with open(bundle.content / "file.raw", "wb") as f:
        f.write(artifact_data)
    bundle.build()

    out, err, exitcode = run(f"rauc install {bundle.output}")
    assert exitcode == 0

    bundle.output.unlink()


def do_install_tree(tmp_path, name, repo_name, artifact_name, artifact_contents):
    bundle = Bundle(tmp_path, name)
    bundle.manifest[f"image.{repo_name}/{artifact_name}"] = {
        "filename": "tree.tar",
    }
    make_tarfile(bundle.content / "tree.tar", artifact_contents)
    bundle.build()

    out, err, exitcode = run(f"rauc install {bundle.output}")
    assert exitcode == 0

    bundle.output.unlink()


def do_install_composefs(tmp_path, name, repo_name, artifact_name, artifact_contents):
    bundle = Bundle(tmp_path, name)
    bundle.manifest[f"image.{repo_name}/{artifact_name}"] = {
        "filename": "tree.tar",
        "convert": "composefs",
    }
    make_tarfile(bundle.content / "tree.tar", artifact_contents)
    bundle.build()

    out, err, exitcode = run(f"rauc install {bundle.output}")
    assert exitcode == 0

    bundle.output.unlink()


def test_file_install(rauc_dbus_service_with_system, tmp_path):
    status = get_status()
    assert set(status.repos.keys()) == {"files", "trees"}
    assert set(status.repos["files"].artifacts) == set()
    assert set(status.repos["trees"].artifacts) == set()

    # install one file artifact and check result
    data_a = b"content-a"
    do_install_file(tmp_path, "a", "files", "artifact-1", data_a)

    status = get_status()
    assert "files" in status.repos
    repo = status.repos["files"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    assert artifact_path.is_symlink()
    with open(artifact_path, "rb") as f:
        assert f.read() == data_a
    assert artifact_path.samefile(Path("/run/rauc/artifacts/files/artifact-1"))

    # update one file artifact and check result
    data_b = b"content-b"
    do_install_file(tmp_path, "b", "files", "artifact-1", data_b)

    status = get_status()
    assert "files" in status.repos
    repo = status.repos["files"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    run_tree(repo.path)
    assert artifact_path.is_symlink()
    with open(artifact_path, "rb") as f:
        assert f.read() == data_b
    assert artifact_path.samefile(Path("/run/rauc/artifacts/files/artifact-1"))

    # install a different file artifact and check result
    data_c = b"content-c"
    do_install_file(tmp_path, "c", "files", "artifact-2", data_c)

    status = get_status()
    assert "files" in status.repos
    repo = status.repos["files"]
    assert "artifact-2" in repo.referenced_artifacts
    assert "artifact-1" not in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-2"
    assert artifact_path.is_symlink()
    with open(artifact_path, "rb") as f:
        assert f.read() == data_c
    assert artifact_path.samefile(Path("/run/rauc/artifacts/files/artifact-2"))
    artifact_path = repo.path / "artifact-1"
    assert not artifact_path.exists()
    assert not Path("/run/rauc/artifacts/trees/artifact-1").exists()

    # test status output in shell format
    out, err, exitcode = run("rauc status --output-format=shell")
    assert exitcode == 0
    out = out.splitlines()
    assert "RAUC_REPOS='1 2'" in out
    assert "RAUC_REPO_ARTIFACTS_2='1'" in out
    assert "RAUC_REPO_ARTIFACT_NAME_2_1='artifact-2'" in out
    assert "RAUC_REPO_ARTIFACT_INSTANCES_2_1='1'" in out
    assert "RAUC_REPO_ARTIFACT_INSTANCE_ACTIVE_2_1_1='1'" in out

    # test status output in readable format
    out, err, exitcode = run("rauc status --output-format=readable")
    assert exitcode == 0
    assert "[artifact-2]" in out
    out = out.splitlines()
    assert "  type: trees" in out
    assert "  type: files" in out


def test_tree_install(rauc_dbus_service_with_system, tmp_path):
    status = get_status()
    assert set(status.repos.keys()) == {"files", "trees"}
    assert set(status.repos["files"].artifacts) == set()
    assert set(status.repos["trees"].artifacts) == set()

    # install one tree artifact and check result
    data_a = b"content-a"
    do_install_tree(tmp_path, "a", "trees", "artifact-1", {"file-a": data_a})
    status = get_status()
    assert "trees" in status.repos
    repo = status.repos["trees"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    assert artifact_path.is_symlink()
    with open(artifact_path / "file-a", "rb") as f:
        assert f.read() == data_a
    assert artifact_path.samefile(Path("/run/rauc/artifacts/trees/artifact-1"))

    # update one tree artifact and check result
    data_b = b"content-b"
    do_install_tree(tmp_path, "b", "trees", "artifact-1", {"file-b": data_b})
    status = get_status()
    assert "trees" in status.repos
    repo = status.repos["trees"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    run_tree(repo.path)
    assert artifact_path.is_symlink()
    with open(artifact_path / "file-b", "rb") as f:
        assert f.read() == data_b
    # old file must be gone
    assert not (artifact_path / "file-a").exists()
    assert artifact_path.samefile(Path("/run/rauc/artifacts/trees/artifact-1"))

    # install a different tree artifact and check result
    data_c = b"content-c"
    do_install_tree(tmp_path, "c", "trees", "artifact-2", {"file-a": data_c})
    status = get_status()
    assert "trees" in status.repos
    repo = status.repos["trees"]
    assert "artifact-2" in repo.referenced_artifacts
    assert "artifact-1" not in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-2"
    assert artifact_path.is_symlink()
    with open(artifact_path / "file-a", "rb") as f:
        assert f.read() == data_c
    assert artifact_path.samefile(Path("/run/rauc/artifacts/trees/artifact-2"))
    artifact_path = repo.path / "artifact-1"
    assert not artifact_path.exists()
    assert not Path("/run/rauc/artifacts/trees/artifact-1").exists()


def test_install_keep_other(rauc_dbus_service_with_system, tmp_path):
    """
    When we install to one repo, artifacts in other repos should not be
    removed.
    """
    status = get_status()
    assert set(status.repos.keys()) == {"files", "trees"}
    assert set(status.repos["files"].artifacts) == set()
    assert set(status.repos["trees"].artifacts) == set()

    # install one file artifact and check result
    data_a = b"content-a"
    do_install_file(tmp_path, "a", "files", "artifact-1", data_a)

    status = get_status()
    assert "files" in status.repos
    repo = status.repos["files"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    assert artifact_path.is_symlink()
    with open(artifact_path, "rb") as f:
        assert f.read() == data_a
    assert artifact_path.samefile(Path("/run/rauc/artifacts/files/artifact-1"))

    # install one tree artifact and check result
    data_b = b"content-b"
    do_install_tree(tmp_path, "b", "trees", "artifact-1", {"file-a": data_b})
    status = get_status()
    assert "trees" in status.repos
    repo = status.repos["trees"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    assert artifact_path.is_symlink()
    with open(artifact_path / "file-a", "rb") as f:
        assert f.read() == data_b
    assert artifact_path.samefile(Path("/run/rauc/artifacts/trees/artifact-1"))

    # the file artifact should not be removed
    repo = status.repos["files"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    assert artifact_path.is_symlink()
    assert artifact_path.samefile(Path("/run/rauc/artifacts/files/artifact-1"))


def test_tree_in_use(rauc_dbus_service_with_system, tmp_path):
    status = get_status()
    assert set(status.repos.keys()) == {"files", "trees"}
    assert set(status.repos["files"].artifacts) == set()
    assert set(status.repos["trees"].artifacts) == set()

    # install one tree artifact and check result
    data_a = b"content-a"
    do_install_tree(tmp_path, "a", "trees", "artifact-1", {"file-a": data_a})
    status = get_status()
    assert "trees" in status.repos
    repo = status.repos["trees"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    assert artifact_path.is_symlink()
    with open(artifact_path / "file-a", "rb") as f:
        assert f.read() == data_a
    assert artifact_path.samefile(Path("/run/rauc/artifacts/trees/artifact-1"))

    # open a file from this artifact and remember the full path
    active_file_path = (artifact_path / "file-a").resolve()
    active_file = open(active_file_path, "rb")

    # install another tree artifact and check result
    data_b = b"content-b"
    do_install_tree(tmp_path, "b", "trees", "artifact-2", {"file-b": data_b})
    status = get_status()
    assert "trees" in status.repos
    repo = status.repos["trees"]
    assert "artifact-1" not in repo.referenced_artifacts
    assert "artifact-2" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-2"
    run_tree(repo.path)
    assert artifact_path.is_symlink()
    with open(artifact_path / "file-b", "rb") as f:
        assert f.read() == data_b
    # old file must be gone
    assert not (artifact_path / "file-a").exists()
    assert artifact_path.samefile(Path("/run/rauc/artifacts/trees/artifact-2"))

    active_file.close()
    with open(active_file_path, "rb") as f:
        assert f.read() == data_a


@needs_composefs
def test_composefs_install(rauc_dbus_service_with_system_composefs, tmp_path):
    status = get_status()
    assert "composefs" in status.repos
    assert set(status.repos["composefs"].artifacts) == set()

    # install one composefs artifact and check result
    data_a = b"content-a" * 1024
    do_install_composefs(tmp_path, "a", "composefs", "artifact-1", {"file-a": data_a})
    status = get_status()
    assert "composefs" in status.repos
    repo = status.repos["composefs"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    run_tree(repo.path)
    assert artifact_path.is_symlink()
    with mounted_composefs(tmp_path, artifact_path / "image.cfs", repo.path / ".rauc-cfs-store") as mount_path:
        run_tree(mount_path)
        with open(mount_path / "file-a", "rb") as f:
            assert f.read() == data_a
    assert artifact_path.samefile(Path("/run/rauc/artifacts/composefs/artifact-1"))

    # update one composefs artifact and check result
    data_b = b"content-b" * 1024
    do_install_composefs(tmp_path, "b", "composefs", "artifact-1", {"file-b": data_b})
    status = get_status()
    assert "composefs" in status.repos
    repo = status.repos["composefs"]
    assert "artifact-1" in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-1"
    run_tree(repo.path)
    assert artifact_path.is_symlink()
    with mounted_composefs(tmp_path, artifact_path / "image.cfs", repo.path / ".rauc-cfs-store") as mount_path:
        run_tree(mount_path)
        with open(mount_path / "file-b", "rb") as f:
            assert f.read() == data_b
    # old file must be gone
    assert not (artifact_path / "file-a").exists()
    assert artifact_path.samefile(Path("/run/rauc/artifacts/composefs/artifact-1"))

    # install a different composefs artifact and check result
    data_c = b"content-c" * 1024
    data_d = b"content-d" * 1024  # fetch multiple objects to exercise sorting
    do_install_composefs(
        tmp_path,
        "c",
        "composefs",
        "artifact-2",
        {
            "file-a": data_c,
            "file-b": data_b,
            "file-d": data_d,
        },
    )
    status = get_status()
    assert "composefs" in status.repos
    repo = status.repos["composefs"]
    assert "artifact-2" in repo.referenced_artifacts
    assert "artifact-1" not in repo.referenced_artifacts

    artifact_path = repo.path / "artifact-2"
    run_tree(repo.path)
    assert artifact_path.is_symlink()
    with mounted_composefs(tmp_path, artifact_path / "image.cfs", repo.path / ".rauc-cfs-store") as mount_path:
        run_tree(mount_path)
        with open(mount_path / "file-a", "rb") as f:
            assert f.read() == data_c
    assert artifact_path.samefile(Path("/run/rauc/artifacts/composefs/artifact-2"))
    artifact_path = repo.path / "artifact-1"
    assert not artifact_path.exists()
    assert not Path("/run/rauc/artifacts/composefs/artifact-1").exists()



#!/usr/bin/env python3

import argparse


def extract_cms(bundle_path):
    with open(bundle_path, "rb") as bundle_file:
        # Move to the end of the bundle to read the last 8 bytes for the CMS length
        bundle_file.seek(-8, 2)
        cms_length_bytes = bundle_file.read(8)

        # Convert the 8 bytes to an integer representing the CMS length
        cms_length = int.from_bytes(cms_length_bytes, "big")
        print(f"CMS length is {cms_length} bytes.")

        # Calculate the starting position of the CMS
        cms_start = bundle_file.tell() - cms_length - 8

        # Seek to the start of the CMS
        bundle_file.seek(cms_start)

        # Read the CMS
        cms = bundle_file.read(cms_length)

        return cms


def main():
    parser = argparse.ArgumentParser(description="extract the CMS from a RAUC bundle")
    parser.add_argument("input", type=str, help="input bundle")
    parser.add_argument("output", type=str, help="output filename")

    args = parser.parse_args()

    # Read from the bundle
    cms = extract_cms(args.input)

    # Write to a new file
    with open(args.output, "wb") as cms_file:
        cms_file.write(cms)

    msg = f"""CMS written to '{args.output}'. You can now...

    print the CMS data structure:
    $ openssl cms -cmsout -in {args.output} -inform DER -print

    skip the signature verification and print the manifest (verity format):
    $ openssl cms -verify -in {args.output} -inform DER -noverify

    verify the signature and print the manifest (verity format):
    $ openssl cms -verify -in {args.output} -inform DER -CAfile <your_ca.pem>

    decrypt, verify and print the manifest (crypt format):
    $ openssl cms -decrypt -in {args.output} -inform DER -inkey <your_key.pem> |
      openssl cms -verify -inform DER -CAfile <your_ca.pem>
    """.rstrip()
    print(msg)


if __name__ == "__main__":
    main()



import shutil

from conftest import have_faketime
from helper import run
from decode_cms import decode_cms


# each signer cert's issuer + serial
SIGNERS = {
    "release-1": ("O=Test Org, CN=Test Org Provisioning CA Release", 1),
    "release-2018": ("O=Test Org, CN=Test Org Provisioning CA Release", 2),
    "autobuilder-1": ("O=Test Org, CN=Test Org Provisioning CA Development", 1),
    "autobuilder-2": ("O=Test Org, CN=Test Org Provisioning CA Development", 2),
}


def get_cms(bundle_path):
    with open(bundle_path, "rb") as bundle_file:
        # read CMS/signature size from bundle (8 bytes)
        bundle_file.seek(-8, 2)
        cms_length_bytes = bundle_file.read(8)
        # calculate CMS offset and length
        cms_length = int.from_bytes(cms_length_bytes, "big")
        cms_start = bundle_file.tell() - cms_length - 8
        # read CMS data from bundle file
        bundle_file.seek(cms_start)
        cms_data = bundle_file.read(cms_length)

    return decode_cms(cms_data)


# for convenience, support both a bytes object or a file path
def get_signers(cms_or_path):
    if isinstance(cms_or_path, dict):
        cms = cms_or_path
    else:
        cms = get_cms(cms_or_path)
    assert cms["contentType"] == "signedData"

    # support multiple signers by collecting into a set
    cms_signers = {(si["issuer"], si["serial"]) for si in cms["signerInfos"]}

    return cms_signers


def get_cert_subjects(cms_or_path):
    if isinstance(cms_or_path, dict):
        cms = cms_or_path
    else:
        cms = get_cms(cms_or_path)
    assert cms["contentType"] == "signedData"

    cms_subjects = {cert["subject"] for cert in cms["certs"]}

    return cms_subjects


def test_resign(tmp_path):
    # copy to tmp path for safe ownership check
    in_bundle = tmp_path / "in.raucb"
    shutil.copyfile("good-bundle.raucb", in_bundle)
    assert get_signers(in_bundle) == {SIGNERS["release-1"]}

    out_bundle = tmp_path / "out.raucb"
    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" resign {in_bundle} {out_bundle}"
        " --signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert out_bundle.exists()
    assert get_signers(out_bundle) == {SIGNERS["autobuilder-1"]}

    out, err, exitcode = run(f"rauc --keyring openssl-ca/rel-ca.pem info {out_bundle}")
    assert exitcode == 1

    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-only-ca.pem info {out_bundle}")
    assert exitcode == 0


def test_resign_verity(tmp_path):
    # copy to tmp path for safe ownership check
    in_bundle = tmp_path / "in.raucb"
    shutil.copyfile("good-verity-bundle.raucb", in_bundle)
    assert get_signers(in_bundle) == {SIGNERS["release-1"]}

    out_bundle = tmp_path / "out.raucb"
    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" resign {in_bundle} {out_bundle}"
        " --signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert out_bundle.exists()
    assert get_signers(out_bundle) == {SIGNERS["autobuilder-1"]}

    out, err, exitcode = run(f"rauc --keyring openssl-ca/rel-ca.pem info {out_bundle}")
    assert exitcode == 1

    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-only-ca.pem info {out_bundle}")
    assert exitcode == 0


def test_resign_append(tmp_path):
    # copy to tmp path for safe ownership check
    in_bundle = tmp_path / "in.raucb"
    shutil.copyfile("good-bundle.raucb", in_bundle)
    assert get_signers(in_bundle) == {SIGNERS["release-1"]}

    out_bundle = tmp_path / "out.raucb"
    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" resign {in_bundle} {out_bundle}"
        " --append"
        " --signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert out_bundle.exists()
    assert get_signers(out_bundle) == {SIGNERS["release-1"], SIGNERS["autobuilder-1"]}

    out, err, exitcode = run(f"rauc --keyring openssl-ca/rel-ca.pem info {out_bundle}")
    assert exitcode == 1
    assert "unable to get local issuer certificate" in err

    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-only-ca.pem info {out_bundle}")
    assert exitcode == 1
    assert "unable to get local issuer certificate" in err

    # verification with multiple signatures is not supported yet
    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info {out_bundle}")
    assert exitcode == 1
    assert "Unsupported number of signers: 2" in err


def test_resign_verity_append(tmp_path):
    # copy to tmp path for safe ownership check
    in_bundle = tmp_path / "in.raucb"
    shutil.copyfile("good-verity-bundle.raucb", in_bundle)
    assert get_signers(in_bundle) == {SIGNERS["release-1"]}

    out_bundle = tmp_path / "out.raucb"
    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" resign {in_bundle} {out_bundle}"
        " --append"
        " --signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert out_bundle.exists()
    assert get_signers(out_bundle) == {SIGNERS["release-1"], SIGNERS["autobuilder-1"]}

    out, err, exitcode = run(f"rauc --keyring openssl-ca/rel-ca.pem info {out_bundle}")
    assert exitcode == 1
    assert "unable to get local issuer certificate" in err

    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-only-ca.pem info {out_bundle}")
    assert exitcode == 1
    assert "unable to get local issuer certificate" in err

    # verification with multiple signatures is not supported yet
    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-ca.pem info {out_bundle}")
    assert exitcode == 1
    assert "Unsupported number of signers: 2" in err


def test_resign_crypt(tmp_path):
    # copy to tmp path for safe ownership check
    in_bundle = tmp_path / "in.raucb"
    shutil.copyfile("good-crypt-bundle-unencrypted.raucb", in_bundle)
    assert get_signers(in_bundle) == {SIGNERS["autobuilder-1"]}

    out_bundle = tmp_path / "out.raucb"
    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/rel/release-1.cert.pem"
        " --key openssl-ca/rel/private/release-1.pem"
        " --keyring openssl-ca/dev-only-ca.pem"
        f" resign {in_bundle} {out_bundle}"
        " --signing-keyring openssl-ca/rel-ca.pem"
    )
    assert exitcode == 0
    assert out_bundle.exists()
    assert get_signers(out_bundle) == {SIGNERS["release-1"]}

    out, err, exitcode = run(f"rauc --keyring openssl-ca/dev-only-ca.pem info {out_bundle}")
    assert exitcode == 1

    out, err, exitcode = run(f"rauc --keyring openssl-ca/rel-ca.pem info {out_bundle}")
    assert exitcode == 0


def test_resign_output_exists(tmp_path):
    # copy to tmp path for safe ownership check
    in_bundle = tmp_path / "in.raucb"
    shutil.copyfile("good-bundle.raucb", in_bundle)

    out_bundle = tmp_path / "out.raucb"
    out_bundle.touch(exist_ok=False)

    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/dev-ca.pem"
        f" resign {in_bundle} {out_bundle}"
    )

    assert exitcode == 1
    assert "already exists" in err

    assert out_bundle.exists()


@have_faketime
def test_resign_extend_not_expired(tmp_path):
    out1_bundle = tmp_path / "out1.raucb"

    out, err, exitcode = run(
        'faketime "2018-01-01"'
        " rauc"
        " --cert openssl-ca/rel/release-2018.cert.pem"
        " --key openssl-ca/rel/private/release-2018.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" bundle install-content {out1_bundle}"
    )
    assert exitcode == 0

    assert out1_bundle.exists()

    out2_bundle = tmp_path / "out2.raucb"

    out, err, exitcode = run(
        'faketime "2018-10-01"'
        " rauc"
        " --cert openssl-ca/rel/release-1.cert.pem"
        " --key openssl-ca/rel/private/release-1.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" resign {out1_bundle} {out2_bundle}"
    )
    assert exitcode == 0

    assert out2_bundle.exists()


@have_faketime
def test_resign_extend_expired_no_verify(tmp_path):
    out1_bundle = tmp_path / "out1.raucb"

    out, err, exitcode = run(
        'faketime "2018-01-01"'
        " rauc"
        " --cert openssl-ca/rel/release-2018.cert.pem"
        " --key openssl-ca/rel/private/release-2018.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" bundle install-content {out1_bundle}"
    )
    assert exitcode == 0
    assert out1_bundle.exists()

    cms = get_cms(out1_bundle)
    assert len(cms["signerInfos"]) == 1
    assert cms["signerInfos"][0]["issuer"] == SIGNERS["release-2018"][0]
    assert cms["signerInfos"][0]["serial"] == SIGNERS["release-2018"][1]
    signing_time = cms["signerInfos"][0]["signedAttrs"]["signingTime"]
    assert signing_time[0] == "utcTime"
    assert signing_time[1].year == 2018
    assert signing_time[1].month == 1
    assert signing_time[1].day == 1

    out2_bundle = tmp_path / "out2.raucb"

    out, err, exitcode = run(
        'faketime "2020-10-01"'
        " rauc"
        " --cert openssl-ca/rel/release-1.cert.pem"
        " --key openssl-ca/rel/private/release-1.pem"
        " --no-verify "
        f" resign {out1_bundle} {out2_bundle}"
    )
    assert exitcode == 0

    assert out2_bundle.exists()

    cms = get_cms(out2_bundle)
    assert len(cms["signerInfos"]) == 1
    assert cms["signerInfos"][0]["issuer"] == SIGNERS["release-1"][0]
    assert cms["signerInfos"][0]["serial"] == SIGNERS["release-1"][1]
    signing_time = cms["signerInfos"][0]["signedAttrs"]["signingTime"]
    assert signing_time[0] == "utcTime"
    assert signing_time[1].year == 2020
    assert signing_time[1].month == 10
    assert signing_time[1].day == 1


def test_resign_append_intermediate(tmp_path):
    # copy to tmp path for safe ownership check
    in_bundle = tmp_path / "in.raucb"
    shutil.copyfile("good-verity-bundle.raucb", in_bundle)
    cms = get_cms(in_bundle)
    assert get_signers(cms) == {SIGNERS["release-1"]}
    assert get_cert_subjects(cms) == {"O=Test Org, CN=Test Org Release-1"}

    out_bundle = tmp_path / "out.raucb"
    out, err, exitcode = run(
        "rauc"
        " --cert openssl-ca/dev/autobuilder-1.cert.pem"
        " --key openssl-ca/dev/private/autobuilder-1.pem"
        " --keyring openssl-ca/rel-ca.pem"
        f" resign {in_bundle} {out_bundle}"
        " --intermediate openssl-ca/dev/ca.cert.pem"
        " --append"
        " --signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert out_bundle.exists()
    cms = get_cms(out_bundle)
    assert get_signers(cms) == {SIGNERS["release-1"], SIGNERS["autobuilder-1"]}
    assert get_cert_subjects(cms) == {
        "O=Test Org, CN=Test Org Release-1",
        "O=Test Org, CN=Test Org Autobuilder-1",
        "O=Test Org, CN=Test Org Provisioning CA Development",
    }

    # no path for the signature by Release-1
    out, err, exitcode = run(f"rauc --keyring openssl-ca/root-ca.pem info {out_bundle}")
    assert exitcode == 1
    assert "unable to get local issuer certificate" in err



import json
import os

from conftest import have_grub, have_qemu, no_service
from helper import run
from helper import slot_data_from_json


@no_service
@have_grub
def test_status_mark_good_internally(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status mark-good")

    assert exitcode == 0


@no_service
@have_grub
def test_status_mark_bad_internally(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status mark-bad")

    assert exitcode == 0


@no_service
@have_grub
def test_status_mark_active_internally(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status mark-active")

    assert exitcode == 0


@no_service
@have_grub
def test_status_mark_good_booted(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status mark-good booted")

    assert exitcode == 0


@no_service
@have_grub
def test_status_mark_good_other(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status mark-good other")

    assert exitcode == 0


@no_service
@have_grub
def test_status_mark_good_any_bootslot(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status mark-good rescue.0")

    assert exitcode == 0


@no_service
@have_grub
def test_status_mark_good_non_bootslot(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} --override-boot-slot=A status mark-good bootloader.0")

    assert exitcode == 1


@have_grub
def test_status_mark_bad_other(rauc_dbus_service_with_system_abc):
    out, err, exitcode = run("rauc status --output-format=json")
    assert exitcode == 0
    status = json.loads(out)

    for slotname, property in status["slots"][0].items():
        if property["state"] != "booted" and property["class"] == "rootfs":
            assert property["boot_status"] == "good"

    out, err, exitcode = run("rauc status mark-bad other")
    assert exitcode == 0

    out, err, exitcode = run("rauc status --output-format=json")
    assert exitcode == 0
    status = json.loads(out)

    for slotname, property in status["slots"][0].items():
        if property["state"] != "booted" and property["class"] == "rootfs":
            assert property["boot_status"] == "bad"


@have_grub
def test_status_mark_prevent_late_fallback(tmp_path, create_system_files, system):
    system.prepare_abc_config()
    system.config["system"]["prevent-late-fallback"] = "true"
    system.write_config()
    with system.running_service("A"):
        out, err, exitcode = run("rauc status --output-format=json")
        assert exitcode == 0
        status = json.loads(out)

        for slotname, property in status["slots"][0].items():
            if property["state"] != "booted" and property["class"] == "rootfs":
                assert property["boot_status"] == "good"

        out, err, exitcode = run("rauc status mark-good")
        assert exitcode == 0

        out, err, exitcode = run("rauc status --output-format=json")
        assert exitcode == 0
        status = json.loads(out)

        for slotname, property in status["slots"][0].items():
            if property["state"] != "booted" and property["class"] == "rootfs":
                assert property["boot_status"] == "bad"


@have_grub
def test_status_mark_good_dbus(rauc_dbus_service_with_system):
    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0
    status_data = json.loads(out)

    assert slot_data_from_json(status_data, "rootfs.0")["boot_status"] == "good"
    assert slot_data_from_json(status_data, "rootfs.1")["boot_status"] == "bad"

    out, err, exitcode = run("rauc status mark-good")

    assert exitcode == 0
    assert "marked slot(s) rootfs.0 as good" in out

    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0
    status_data = json.loads(out)

    assert slot_data_from_json(status_data, "rootfs.0")["boot_status"] == "good"
    assert slot_data_from_json(status_data, "rootfs.1")["boot_status"] == "bad"


@have_grub
def test_status_mark_bad_dbus(rauc_dbus_service_with_system):
    # check pre-condition
    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0
    status_data = json.loads(out)

    assert slot_data_from_json(status_data, "rootfs.0")["boot_status"] == "good"
    assert slot_data_from_json(status_data, "rootfs.1")["boot_status"] == "bad"

    # mark bad
    out, err, exitcode = run("rauc status mark-bad")

    assert exitcode == 0
    assert "marked slot(s) rootfs.0 as bad" in out

    # check post-condition
    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0
    status_data = json.loads(out)

    assert slot_data_from_json(status_data, "rootfs.0")["boot_status"] == "bad"
    assert slot_data_from_json(status_data, "rootfs.1")["boot_status"] == "bad"


@have_grub
def test_status_mark_active_dbus(rauc_dbus_service_with_system):
    # check pre-condition
    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0
    status_data = json.loads(out)

    assert slot_data_from_json(status_data, "rootfs.0")["boot_status"] == "good"
    assert slot_data_from_json(status_data, "rootfs.1")["boot_status"] == "bad"
    assert status_data["boot_primary"] == "rootfs.0"

    # mark active other
    out, err, exitcode = run("rauc status mark-active other")

    assert exitcode == 0
    assert "rootfs.1 as active" in out

    # check post-condition
    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0
    status_data = json.loads(out)

    assert slot_data_from_json(status_data, "rootfs.0")["boot_status"] == "good"
    assert slot_data_from_json(status_data, "rootfs.1")["boot_status"] == "good"
    assert status_data["boot_primary"] == "rootfs.1"


@have_qemu
def test_status_mark_bad_barebox(tmp_path, create_system_files, system):
    """
    Tests that 'mark-bad' call for barebox alters the barebox state
    variables appropriately.

    Leverages the 'barebox-state' dummy with variables pre-set from env.
    """

    system.prepare_minimal_config()
    system.config["system"]["bootloader"] = "barebox"
    del system.config["system"]["grubenv"]
    system.write_config()
    os.environ["BAREBOX_STATE_VARS_PRE"] = """
bootstate.A.priority=20
bootstate.B.priority=21
bootstate.A.remaining_attempts=3
bootstate.B.remaining_attempts=3
"""
    os.environ["BAREBOX_STATE_VARS_POST"] = """
bootstate.A.priority=20
bootstate.B.priority=0
bootstate.A.remaining_attempts=3
bootstate.B.remaining_attempts=0
"""
    with system.running_service("A"):
        # mark rootfs.1 (B) bad
        out, err, exitcode = run("rauc status mark-bad rootfs.1")
        assert not err
        assert exitcode == 0


@have_qemu
def test_status_mark_active_barebox(tmp_path, create_system_files, system):
    """
    Tests that 'mark-primary' call for barebox alters the barebox state
    variables appropriately.

    Leverages the 'barebox-state' dummy with variables pre-set from env.
    """

    system.prepare_minimal_config()
    system.config["system"]["bootloader"] = "barebox"
    del system.config["system"]["grubenv"]
    system.write_config()
    os.environ["BAREBOX_STATE_VARS_PRE"] = """
bootstate.A.priority=21
bootstate.B.priority=20
bootstate.A.remaining_attempts=3
bootstate.B.remaining_attempts=0
"""
    os.environ["BAREBOX_STATE_VARS_POST"] = """
bootstate.A.priority=10
bootstate.B.priority=20
bootstate.A.remaining_attempts=3
bootstate.B.remaining_attempts=3
"""
    with system.running_service("A"):
        # mark rootfs.1 (B) active/primary
        out, err, exitcode = run("rauc status mark-active rootfs.1")
        assert not err
        assert exitcode == 0



import json
import uuid
from pathlib import Path

import pytest
from dasbus.typing import get_native, get_variant

from conftest import have_json, have_service
from helper import run


def test_backend_range(http_server):
    """Test if the backend returns the range parameters correctly."""
    http_server.setup(
        file_path="good-verity-bundle.raucb",
    )

    resp = http_server.get(headers={"Range": "bytes=0-3"})
    resp.raise_for_status()
    assert resp.status_code == 206
    assert resp.content == b"hsqs"

    summary = http_server.get_summary()
    assert summary["requests"] == 1
    assert summary["range_requests"] == ["0:4"]


def test_backend_headers(http_server):
    """Test if the backend returns the request headers correctly."""
    http_server.setup(
        file_path="good-verity-bundle.raucb",
    )

    resp = http_server.head(headers={"RAUC-Test": "value"})
    resp.raise_for_status()
    assert resp.status_code == 200

    summary = http_server.get_summary()
    assert summary["requests"] == 1
    assert summary["first_request_headers"].get("RAUC-Test") == "value"


def prune_standard_headers(headers):
    for k in ["Host", "X-Forwarded-For", "Connection", "Accept", "User-Agent"]:
        try:
            del headers[k]
        except KeyError:
            pass


def is_uuid(value):
    try:
        uuid.UUID(value)
    except ValueError:
        return False
    return True


def is_uptime(value):
    try:
        float(value)
    except ValueError:
        return False
    return True


@pytest.mark.parametrize("api", ["cli", "dbus"])
@have_json
def test_info_headers(create_system_files, system, http_server, api):
    """Test if the info command sends custom headers correctly."""
    if api == "dbus" and not have_service():
        pytest.skip("Missing service")

    system.prepare_minimal_config()
    system.config["handlers"] = {
        "system-info": "bin/systeminfo.sh",
    }
    system.config["streaming"] = {
        "send-headers": "boot-id;machine-id;system-version;serial;variant;transaction-id;uptime",
    }
    system.write_config()
    http_server.setup(
        file_path="good-verity-bundle.raucb",
    )

    if api == "cli":
        out, err, exitcode = run(
            f"{system.prefix} info {http_server.url} --output-format=json -H 'Test-Header: Test-Value'"
        )
        assert exitcode == 0
        info = json.loads(out)
        assert info["compatible"] == "Test Config"
    elif api == "dbus":
        with system.running_service("A"):
            info = system.proxy.InspectBundle(
                http_server.url,
                {
                    "http-headers": get_variant("as", ["Test-Header: Test-Value"]),
                },
            )
        info = get_native(info)
        assert info["update"]["compatible"] == "Test Config"

    summary = http_server.get_summary()
    assert summary["requests"] == 3

    first_headers = summary["first_request_headers"]
    assert first_headers.pop("User-Agent").startswith("rauc/")
    assert is_uuid(first_headers.pop("RAUC-Boot-ID"))
    machine_id = Path("/etc/machine-id")
    if machine_id.exists() and machine_id.stat().st_size > 0:
        assert is_uuid(first_headers.pop("RAUC-Machine-ID"))
    else:
        assert "RAUC-Machine-ID" not in first_headers
    assert is_uptime(first_headers.pop("RAUC-Uptime"))
    prune_standard_headers(first_headers)
    assert first_headers == {
        "Range": "bytes=0-3",
        "Test-Header": "Test-Value",
        "RAUC-Serial": "1234",
        "RAUC-System-Version": "1.0.0",
        "RAUC-Variant": "test-variant-x",
    }

    second_headers = summary["second_request_headers"]
    prune_standard_headers(second_headers)
    assert second_headers == {
        "Range": "bytes=26498-26505",
        "Test-Header": "Test-Value",
    }

    assert summary["range_requests"] == [
        "0:4",  # magic
        "26498:26506",  # CMS size
        "24576:26498",  # CMS data
    ]



#!/usr/bin/env python3

import tarfile

from io import BytesIO

# create a tar file with some normal files
with tarfile.open(name="payload-common.tar", mode="w") as t:
    f = tarfile.TarInfo(name="file")
    t.addfile(f)

    c = BytesIO(b"contents")
    f = tarfile.TarInfo(name="file-contents")
    f.size = len(c.getbuffer())
    t.addfile(f, c)

    f = tarfile.TarInfo(name="file-user")
    f.uid = 1000
    f.gid = 1000
    t.addfile(f)

    f = tarfile.TarInfo(name="file-mtime")
    f.mtime = 1694006436.1317987
    t.addfile(f)

    f = tarfile.TarInfo(name="executable")
    f.mode = 0o755
    t.addfile(f)

    f = tarfile.TarInfo(name="dir")
    f.type = tarfile.DIRTYPE
    f.mode = 0o750
    t.addfile(f)

    f = tarfile.TarInfo(name="dir/file")
    t.addfile(f)

    f = tarfile.TarInfo(name="symlink")
    f.type = tarfile.SYMTYPE
    f.linkname = "./dir/file"
    t.addfile(f)

    f = tarfile.TarInfo(name="broken-symlink")
    f.type = tarfile.SYMTYPE
    f.linkname = "/nonexistent/symlink/target"
    t.addfile(f)

    f = tarfile.TarInfo(name="hardlink")
    f.type = tarfile.LNKTYPE
    f.linkname = "file"
    t.addfile(f)

    f = tarfile.TarInfo(name="devchr")
    f.type = tarfile.CHRTYPE
    # /dev/null
    f.devmajor = 1
    f.devminor = 3
    t.addfile(f)

    f = tarfile.TarInfo(name="devblk")
    f.type = tarfile.BLKTYPE
    # /dev/dm-0
    f.devmajor = 253
    f.devminor = 0
    t.addfile(f)

    f = tarfile.TarInfo(name="fifo")
    f.type = tarfile.FIFOTYPE
    t.addfile(f)

# create a tar file with some special cases
with tarfile.open(name="payload-special.tar", mode="w") as t:
    f = tarfile.TarInfo(name="file")
    t.addfile(f)

    f = tarfile.TarInfo(name="acl")
    f.pax_headers["SCHILY.acl.access"] = "user::rw-\nuser:root:r--\ngroup::r--\nmask::r--\nother::r--\n"
    t.addfile(f)

    f = tarfile.TarInfo(name="xattr")
    f.pax_headers["SCHILY.xattr.user.foo"] = "bar"
    t.addfile(f)

    f = tarfile.TarInfo(name="selinux")
    f.pax_headers["RHT.security.selinux"] = "system_u:object_r:dummy_t"
    t.addfile(f)

# Generate two tars which use the same contents multiple times in different
# combinations, which is useful for artifact testing.
CONTENTS_A = b"contents-a\n" * 10000
CONTENTS_B = b"contents-b\n" * 10000
CONTENTS_C = b"contents-c\n" * 10000
CONTENTS_D = b"contents-d\n" * 10000


def add_file(tar, name, contents, uid=0):
    c = BytesIO(contents)
    f = tarfile.TarInfo(name=name)
    f.size = len(c.getbuffer())
    f.uid = uid
    tar.addfile(f, c)


with tarfile.open(name="payload-medium-data-size-a.tar.gz", mode="w:gz") as t:
    add_file(t, "file-a-1", CONTENTS_A)
    add_file(t, "file-a-2", CONTENTS_A, uid=1000)
    add_file(t, "file-b-1", CONTENTS_B)
    add_file(t, "file-b-2", CONTENTS_B)
    add_file(t, "file-c-1", CONTENTS_C)

    f = tarfile.TarInfo(name="file-a-1.symlink")
    f.type = tarfile.SYMTYPE
    f.linkname = "./file-a-1"
    t.addfile(f)

    f = tarfile.TarInfo(name="file-a-1.hardlink")
    f.type = tarfile.LNKTYPE
    f.linkname = "file-a-1"
    t.addfile(f)


with tarfile.open(name="payload-medium-data-size-b.tar.gz", mode="w:gz") as t:
    add_file(t, "file-b-1", CONTENTS_B)
    add_file(t, "file-c-1", CONTENTS_C)
    add_file(t, "file-c-2", CONTENTS_C, uid=1000)
    add_file(t, "file-d-1", CONTENTS_D)

    f = tarfile.TarInfo(name="file-b-1.symlink")
    f.type = tarfile.SYMTYPE
    f.linkname = "./file-b-1"
    t.addfile(f)

    f = tarfile.TarInfo(name="file-b-1.hardlink")
    f.type = tarfile.LNKTYPE
    f.linkname = "file-b-1"
    t.addfile(f)



import os
import shutil

from conftest import have_openssl
from helper import run


@have_openssl
def test_replace_signature_plain(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        "rauc "
        "--cert openssl-ca/dev/autobuilder-1.cert.pem "
        "--key openssl-ca/dev/private/autobuilder-1.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"resign {tmp_path}/good-bundle.raucb {tmp_path}/out1.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )

    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out1.raucb")

    out, err, exitcode = run(
        f"rauc --keyring openssl-ca/dev-only-ca.pem extract-signature {tmp_path}/out1.raucb {tmp_path}/bundle.sig"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/bundle.sig")

    out, err, exitcode = run(f"openssl asn1parse -inform DER -in {tmp_path}/bundle.sig -noout")
    assert exitcode == 0

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-ca.pem "
        f"replace-signature {tmp_path}/good-bundle.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/out2.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out2.raucb")


@have_openssl
def test_replace_signature_verity(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-verity-bundle.raucb", tmp_path / "good-verity-bundle.raucb")

    out, err, exitcode = run(
        "rauc "
        "--cert openssl-ca/dev/autobuilder-1.cert.pem "
        "--key openssl-ca/dev/private/autobuilder-1.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"resign {tmp_path}/good-verity-bundle.raucb {tmp_path}/out1.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )

    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out1.raucb")

    out, err, exitcode = run(
        f"rauc --keyring openssl-ca/dev-only-ca.pem extract-signature {tmp_path}/out1.raucb {tmp_path}/bundle.sig"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/bundle.sig")

    out, err, exitcode = run(f"openssl asn1parse -inform DER -in {tmp_path}/bundle.sig -noout")
    assert exitcode == 0

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-ca.pem "
        f"replace-signature {tmp_path}/good-verity-bundle.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/out2.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out2.raucb")


@have_openssl
def test_replace_signature_crypt(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-crypt-bundle-unencrypted.raucb", tmp_path / "good-crypt-bundle-unencrypted.raucb")

    out, err, exitcode = run(
        "rauc "
        "--cert openssl-ca/dev/autobuilder-1.cert.pem "
        "--key openssl-ca/dev/private/autobuilder-1.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"resign {tmp_path}/good-crypt-bundle-unencrypted.raucb {tmp_path}/out1.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )

    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out1.raucb")

    out, err, exitcode = run(
        f"rauc --keyring openssl-ca/dev-only-ca.pem extract-signature {tmp_path}/out1.raucb {tmp_path}/bundle.sig"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/bundle.sig")

    out, err, exitcode = run(f"openssl asn1parse -inform DER -in {tmp_path}/bundle.sig -noout")
    assert exitcode == 0

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-ca.pem "
        f"replace-signature {tmp_path}/good-crypt-bundle-unencrypted.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/out2.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out2.raucb")


@have_openssl
def test_replace_signature_output_exists(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    open(f"{tmp_path}/out2.raucb", "a").close()

    out, err, exitcode = run(
        "rauc "
        "--cert openssl-ca/dev/autobuilder-1.cert.pem "
        "--key openssl-ca/dev/private/autobuilder-1.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"resign {tmp_path}/good-bundle.raucb {tmp_path}/out1.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )

    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out1.raucb")

    out, err, exitcode = run(
        f"rauc --keyring openssl-ca/dev-only-ca.pem extract-signature {tmp_path}/out1.raucb {tmp_path}/bundle.sig"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/bundle.sig")

    out, err, exitcode = run(f"openssl asn1parse -inform DER -in {tmp_path}/bundle.sig -noout")
    assert exitcode == 0

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-ca.pem "
        f"replace-signature {tmp_path}/good-bundle.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/out2.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 1
    assert os.path.exists(f"{tmp_path}/out2.raucb")


@have_openssl
def test_replace_signature_bad_keyring(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        "rauc "
        "--cert openssl-ca/dev/autobuilder-1.cert.pem "
        "--key openssl-ca/dev/private/autobuilder-1.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"resign {tmp_path}/good-bundle.raucb {tmp_path}/out1.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )

    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out1.raucb")

    out, err, exitcode = run(
        f"rauc --keyring openssl-ca/dev-only-ca.pem extract-signature {tmp_path}/out1.raucb {tmp_path}/bundle.sig"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/bundle.sig")

    out, err, exitcode = run(f"openssl asn1parse -inform DER -in {tmp_path}/bundle.sig -noout")
    assert exitcode == 0

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-only-ca.pem "
        f"replace-signature {tmp_path}/good-bundle.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/out2.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 1
    assert not os.path.exists(f"{tmp_path}/out2.raucb")


@have_openssl
def test_replace_signature_no_verify(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(
        "rauc "
        "--cert openssl-ca/dev/autobuilder-1.cert.pem "
        "--key openssl-ca/dev/private/autobuilder-1.pem "
        "--keyring openssl-ca/dev-ca.pem "
        f"resign {tmp_path}/good-bundle.raucb {tmp_path}/out1.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )

    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out1.raucb")

    out, err, exitcode = run(
        f"rauc --keyring openssl-ca/dev-only-ca.pem extract-signature {tmp_path}/out1.raucb {tmp_path}/bundle.sig"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/bundle.sig")

    out, err, exitcode = run(f"openssl asn1parse -inform DER -in {tmp_path}/bundle.sig -noout")
    assert exitcode == 0

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-only-ca.pem "
        "--no-verify "
        f"replace-signature {tmp_path}/good-bundle.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/out2.raucb "
        "--signing-keyring openssl-ca/dev-only-ca.pem"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/out2.raucb")


@have_openssl
def test_replace_signature_invalid_bundle_signature_output(tmp_path):
    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    open(f"{tmp_path}/invalid.raucb", "a").close()
    open(f"{tmp_path}/invalid.sig", "a").close()

    out, err, exitcode = run(
        f"rauc --keyring openssl-ca/dev-ca.pem extract-signature {tmp_path}/good-bundle.raucb {tmp_path}/bundle.sig"
    )
    assert exitcode == 0
    assert os.path.exists(f"{tmp_path}/bundle.sig")

    out, err, exitcode = run(f"openssl asn1parse -inform DER -in {tmp_path}/bundle.sig -noout")
    assert exitcode == 0

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-ca.pem "
        f"replace-signature {tmp_path}/invalid.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/out.raucb "
        "--signing-keyring openssl-ca/dev-ca.pem"
    )
    assert exitcode == 1
    assert not os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-ca.pem "
        f"replace-signature {tmp_path}/good-bundle.raucb "
        f"{tmp_path}/invalid.sig {tmp_path}/out.raucb "
        "--signing-keyring openssl-ca/dev-ca.pem"
    )
    assert exitcode == 1
    assert not os.path.exists(f"{tmp_path}/out.raucb")

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-ca.pem "
        f"replace-signature {tmp_path}/good-bundle.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/notexisting/out.raucb "
        "--signing-keyring openssl-ca/dev-ca.pem"
    )
    assert exitcode == 1
    assert not os.path.exists(f"{tmp_path}/notexisting/out.raucb")

    out, err, exitcode = run(
        "rauc "
        "--keyring openssl-ca/dev-ca.pem "
        f"replace-signature {tmp_path}/good-bundle.raucb "
        f"{tmp_path}/bundle.sig {tmp_path}/good-bundle.raucb "
        "--signing-keyring openssl-ca/dev-ca.pem"
    )
    assert exitcode == 1
    assert os.path.exists(f"{tmp_path}/good-bundle.raucb")



import os
from subprocess import check_call

from helper import run
from conftest import needs_emmc


def test_write_slot_invalid_local_paths():
    out, err, exitcode = run("rauc -c test.conf write-slot rootfs.0 foo")
    assert exitcode == 1
    assert "No such file or directory" in err

    out, err, exitcode = run("rauc -c test.conf write-slot rootfs.0 foo.raucb")
    assert exitcode == 1
    assert "No such file or directory" in err

    out, err, exitcode = run("rauc -c test.conf write-slot rootfs.0 /path/to/foo.raucb")
    assert exitcode == 1
    assert "No such file or directory" in err


def test_write_slot_invalid_slot():
    out, err, exitcode = run("rauc -c test.conf write-slot dummy install-content/rootfs.img")
    assert exitcode == 1
    assert "No matching slot found for given slot name" in err


def test_write_slot_readonly():
    out, err, exitcode = run("rauc -c test.conf write-slot rescue.0 install-content/appfs.img")
    assert exitcode == 1
    assert "Reject writing to readonly slot" in err


def test_write_slot(rauc_no_service):
    out, err, exitcode = run(f"{rauc_no_service} write-slot rootfs.0 install-content/appfs.img")
    assert exitcode == 0
    assert "Slot written successfully" in out


def test_write_slot_image_type(tmp_path, rauc_no_service):
    """
    Tests writing image with non-standard '.bin' extension to an 'ext4' slot:
    - First run without explicit '--image-type' option set (should fail).
    - Second run *with* this option set (should succeed).
    """
    open(tmp_path / "image.bin", mode="w").close()

    out, err, exitcode = run(f"{rauc_no_service} write-slot rootfs.0 {tmp_path}/image.bin")
    assert exitcode == 1
    assert f"Unable to map extension of file '{tmp_path}/image.bin' to known image type" in err

    out, err, exitcode = run(f"{rauc_no_service} write-slot --image-type=raw rootfs.0 install-content/appfs.img")
    assert exitcode == 0
    assert "Slot written successfully" in out


def test_write_slot_no_handler(tmp_path, rauc_no_service):
    open(tmp_path / "image.vfat", mode="w").close()

    out, err, exitcode = run(f"{rauc_no_service} write-slot rootfs.0 {tmp_path}/image.vfat")
    assert exitcode == 1
    assert "Unsupported image type 'vfat' for slot type 'ext4'" in err


@needs_emmc
def test_write_boot_emmc(system):
    device = os.environ["RAUC_TEST_EMMC"]

    # disable boot partition to have a fixed setup
    check_call(["mmc", "bootpart", "enable", "0", "0", device])

    system.config["slot.bootloader.0"] = {
        "device": device,
        "type": "boot-emmc",
    }
    system.write_config()

    out, err, exitcode = run(f"{system.prefix} write-slot bootloader.0 install-content/rootfs.img")
    assert exitcode == 0
    assert "Slot written successfully" in out
    assert "eMMC device was not enabled for booting, yet. Ignoring." in err
    assert f"Boot partition {device}boot0 is now active" in err

    out, err, exitcode = run(f"{system.prefix} write-slot bootloader.0 install-content/rootfs.img")
    assert exitcode == 0
    assert "Slot written successfully" in out
    assert "Found active eMMC boot partition /dev/mmcblk0boot0" in err
    assert f"Boot partition {device}boot1 is now active" in err


@needs_emmc
def test_write_boot_emmc_size_limit(system):
    """
    Sets 'size-limit' option for boot-emmc slot and checks that after writing,
    the data above the size-limit remains untouched.
    """
    device = os.environ["RAUC_TEST_EMMC"]
    bootdevice = f"{device}boot0"
    size = 1024 * 1024  # full size of eMMC boot partition
    half_size = size // 2

    # disable boot partition to have a fixed setup
    check_call(["mmc", "bootpart", "enable", "0", "0", device])

    system.config["slot.bootloader.0"] = {
        "device": device,
        "type": "boot-emmc",
        "size-limit": f"{half_size}",
    }
    system.write_config()

    # Prepare known data
    original_data = os.urandom(size)
    with open(f"/sys/block/{bootdevice[4:]}/force_ro", "w") as f:
        f.write("0")
    with open(bootdevice, "wb") as f:
        f.write(original_data)
    with open(f"/sys/block/{bootdevice[4:]}/force_ro", "w") as f:
        f.write("1")

    # write image
    out, err, exitcode = run(f"{system.prefix} write-slot bootloader.0 install-content/rootfs.img")
    assert exitcode == 0
    assert "Slot written successfully" in out
    assert "eMMC device was not enabled for booting, yet. Ignoring." in err
    assert f"Boot partition {device}boot0 is now active" in err

    assert f"Cleared first {half_size} bytes on /dev/mmcblk0boot0" in err

    # Read back from device
    with open(bootdevice, "rb") as f:
        result_data = f.read(1024 * 1024)

    # Check first 16 bytes below 512 KiB are zeroed
    assert result_data[half_size - 0x10 : half_size] == b"\x00" * 0x10, "First 512 KiB is not zeroed"

    # Check first 16 bytes above 512 KiB are intact
    assert result_data[half_size : half_size + 0x10] == original_data[half_size : half_size + 0x10], (
        "Second 512 KiB is not intact"
    )


@needs_emmc
def test_write_boot_emmc_size_limit_too_large(system):
    """
    Sets 'size-limit' option for boot-emmc slot to a value larger then the
    actual size of the partition and ensures RAUC prints a warning.
    """
    device = os.environ["RAUC_TEST_EMMC"]

    # disable boot partition to have a fixed setup
    check_call(["mmc", "bootpart", "enable", "0", "0", device])

    system.config["slot.bootloader.0"] = {
        "device": device,
        "type": "boot-emmc",
        "size-limit": "10M",
    }
    system.write_config()

    out, err, exitcode = run(f"{system.prefix} write-slot bootloader.0 install-content/rootfs.img")
    assert exitcode == 0
    assert "Slot written successfully" in out
    assert "eMMC device was not enabled for booting, yet. Ignoring." in err
    assert f"Boot partition {device}boot0 is now active" in err

    assert "The size-limit (10485760 bytes) exceeds actual device size" in err



import json
import os
import shutil
from textwrap import dedent

from conftest import have_casync, have_http, have_streaming, no_service, root
from helper import run
from helper import slot_data_from_json

# all tests require root privileges
pytestmark = root


def test_install(rauc_dbus_service_with_system, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0
    assert os.path.isdir("/run/rauc/slots/active")
    assert os.path.islink("/run/rauc/slots/active/rootfs")
    assert os.readlink("/run/rauc/slots/active/rootfs")

    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0
    status_data = json.loads(out)

    assert slot_data_from_json(status_data, "rootfs.0")["boot_status"] == "good"
    assert slot_data_from_json(status_data, "rootfs.1")["boot_status"] == "bad"
    assert status_data["boot_primary"] == "rootfs.0"

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(f"rauc install {tmp_path}/good-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0

    out, err, exitcode = run("rauc status --output-format=json-pretty")
    assert exitcode == 0
    status_data = json.loads(out)

    assert slot_data_from_json(status_data, "rootfs.0")["boot_status"] == "good"
    assert slot_data_from_json(status_data, "rootfs.1")["boot_status"] == "good"
    assert status_data["boot_primary"] == "rootfs.1"


def test_install_verity(rauc_dbus_service_with_system, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-verity-bundle.raucb", tmp_path / "good-verity-bundle.raucb")

    out, err, exitcode = run(f"rauc install {tmp_path}/good-verity-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


def test_install_crypt(rauc_dbus_service_with_system_crypt, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-crypt-bundle-encrypted.raucb", tmp_path / "good-crypt-bundle-encrypted.raucb")

    out, err, exitcode = run(f"rauc install {tmp_path}/good-crypt-bundle-encrypted.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


def test_install_env(rauc_dbus_service_with_system_adaptive, tmp_path):
    assert (tmp_path / "images/rootfs-1").is_file()
    assert (tmp_path / "images/rootfs-1").stat().st_size == 0

    bundle_name = "good-adaptive-meta-bundle.raucb"

    # copy to tmp path for safe ownership check
    shutil.copyfile(bundle_name, tmp_path / bundle_name)

    out, err, exitcode = run(f"rauc install {tmp_path}/{bundle_name}")

    assert exitcode == 0
    assert (tmp_path / "images/rootfs-1").stat().st_size > 0

    with open(tmp_path / "preinstall-env") as f:
        pre_lines = f.readlines()
        assert "RAUC_CURRENT_BOOTNAME=A\n" in pre_lines
        assert "RAUC_TARGET_SLOTS=1 2\n" in pre_lines
        assert "RAUC_META_UPDATE_POLL=86400\n" in pre_lines
        assert "RAUC_META_VERSION_CHANNEL=beta\n" in pre_lines

    with open(tmp_path / "postinstall-env") as f:
        post_lines = f.readlines()

    assert post_lines == pre_lines


@have_casync
def test_install_plain_casync_local(rauc_dbus_service_with_system, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-casync-bundle-1.5.1.raucb", tmp_path / "good-casync-bundle-1.5.1.raucb")
    shutil.copytree("good-casync-bundle-1.5.1.castr", tmp_path / "good-casync-bundle-1.5.1.castr")

    out, err, exitcode = run(f"rauc install {tmp_path}/good-casync-bundle-1.5.1.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


@have_casync
@have_http
def test_install_verity_casync_http(rauc_dbus_service_with_system, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    out, err, exitcode = run("rauc install http://127.0.0.1/test/good-casync-bundle-verity.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


@have_streaming
def test_install_streaming(rauc_dbus_service_with_system, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    out, err, exitcode = run("rauc install http://127.0.0.1/test/good-verity-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


@have_streaming
def test_install_streaming_error(rauc_dbus_service_with_system, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    out, err, exitcode = run("rauc install http://127.0.0.1/test/missing-bundle.raucb")

    assert exitcode == 1
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0


def test_install_progress(rauc_dbus_service_with_system, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(f"rauc install --progress {tmp_path}/good-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


def test_install_rauc_external(rauc_dbus_service_with_system_external, tmp_path):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    out, err, exitcode = run(f"rauc install {tmp_path}/good-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0
    assert os.path.getsize(tmp_path / "images/rootfs-0") == 0

    out, err, exitcode = run(f"rauc install {tmp_path}/good-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0
    assert os.path.getsize(tmp_path / "images/rootfs-0") > 0


def test_install_per_slot_status(tmp_path, create_system_files, system):
    """Tests that installation works with 'per-slot' status file"""
    system.prepare_minimal_config()
    del system.config["system"]["data-directory"]
    system.config["system"]["statusfile"] = "per-slot"
    system.write_config()

    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-verity-bundle.raucb", tmp_path / "good-verity-bundle.raucb")

    with system.running_service("A"):
        out, err, exitcode = run(f"rauc install {tmp_path}/good-verity-bundle.raucb")

        assert exitcode == 0
        assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


def test_install_abc(rauc_dbus_service_with_system_abc, tmp_path):
    """Tests that two consecutive calls of 'rauc install' update different slot groups"""
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0
    assert os.path.exists(tmp_path / "images/rootfs-2")
    assert not os.path.getsize(tmp_path / "images/rootfs-2") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-verity-bundle.raucb", tmp_path / "good-verity-bundle.raucb")

    # First installation should update rootfs-1 (implementation-defined)
    out, err, exitcode = run(f"rauc install {tmp_path}/good-verity-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-0") == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0
    assert os.path.getsize(tmp_path / "images/rootfs-2") == 0

    # Second installation should update rootfs-2 (implementation-defined)
    out, err, exitcode = run(f"rauc install {tmp_path}/good-verity-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-0") == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0
    assert os.path.getsize(tmp_path / "images/rootfs-2") > 0


@no_service
def test_install_no_service(tmp_path, create_system_files, system):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    system.prepare_minimal_config()
    system.write_config()
    out, err, exitcode = run(f"{system.prefix} install --override-boot-slot=A {tmp_path}/good-bundle.raucb")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


@no_service
@have_streaming
def test_install_no_service_streaming(tmp_path, create_system_files, system):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    system.prepare_minimal_config()
    system.write_config()
    out, err, exitcode = run(
        f"{system.prefix} --override-boot-slot=A install http://127.0.0.1/test/good-verity-bundle.raucb"
    )

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0


@no_service
@have_streaming
def test_install_no_service_streaming_error(tmp_path, create_system_files, system):
    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    # copy to tmp path for safe ownership check
    shutil.copyfile("good-bundle.raucb", tmp_path / "good-bundle.raucb")

    system.prepare_minimal_config()
    system.write_config()
    out, err, exitcode = run(
        f"{system.prefix} --override-boot-slot=A install http://127.0.0.1/test/missing-bundle.raucb"
    )

    assert exitcode == 1
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0


def test_install_hook_env(rauc_dbus_service_with_system, tmp_path, bundle):
    bundle.add_hook_script(
        dedent("""\
    #!/bin/bash
    set -e

    if [ -z "$RAUC_PYTEST_TMP" ]; then
        exit 1
    fi

    case "$1" in
        install-check)
            env | sort > "$RAUC_PYTEST_TMP/install-check-hook-env"
            ;;
        slot-pre-install)
            env | sort > "$RAUC_PYTEST_TMP/slot-pre-install-hook-env"
            ;;
        slot-install)
            env | sort > "$RAUC_PYTEST_TMP/slot-install-hook-env"
            ;;
        slot-post-install)
            env | sort > "$RAUC_PYTEST_TMP/slot-post-install-hook-env"
            ;;
        *)
            exit 1
            ;;
    esac
    """)
    )
    bundle.manifest["hooks"]["hooks"] = "install-check"
    bundle.manifest["image.rootfs"] = {
        "filename": "rootfs.img",
        "hooks": "pre-install;post-install",
    }
    bundle.make_random_image("rootfs", 4096, "random rootfs")
    bundle.manifest["image.appfs"] = {
        "filename": "appfs.img",
        "hooks": "install",
    }
    bundle.make_random_image("appfs", 4096, "random appfs")
    bundle.manifest["meta.test"] = {
        "foo": "bar",
    }
    bundle.build()

    out, err, exitcode = run(f"rauc install {bundle.output}")

    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0

    with open(tmp_path / "install-check-hook-env") as f:
        check_lines = f.readlines()
        assert "RAUC_MF_VERSION=2011.03-2\n" in check_lines
        assert "RAUC_SYSTEM_COMPATIBLE=Test Config\n" in check_lines
        assert "RAUC_SYSTEM_VARIANT=Default Variant\n" in check_lines
        assert "RAUC_META_TEST_FOO=bar\n" in check_lines

    with open(tmp_path / "slot-pre-install-hook-env") as f:
        pre_lines = f.readlines()
        assert "RAUC_IMAGE_CLASS=rootfs\n" in pre_lines
        assert "RAUC_IMAGE_SIZE=4096\n" in pre_lines
        assert "RAUC_SLOT_NAME=rootfs.1\n" in pre_lines
        assert "RAUC_SLOT_STATE=inactive\n" in pre_lines

    with open(tmp_path / "slot-post-install-hook-env") as f:
        post_lines = f.readlines()
        assert post_lines == pre_lines

    with open(tmp_path / "slot-install-hook-env") as f:
        install_lines = f.readlines()
        assert "RAUC_IMAGE_CLASS=appfs\n" in install_lines
        assert "RAUC_IMAGE_SIZE=4096\n" in install_lines
        assert "RAUC_SLOT_NAME=appfs.1\n" in install_lines
        assert "RAUC_SLOT_STATE=inactive\n" in install_lines


@have_streaming
def test_install_require_hash(rauc_dbus_service_with_system, tmp_path):
    GOOD_HASH = "2a7c9b2a31f11575deef280812e714fdb4542b55d308e39e85352ff996d79b8a"
    BAD_HASH = GOOD_HASH[:-1] + "0"

    assert os.path.exists(tmp_path / "images/rootfs-1")
    assert not os.path.getsize(tmp_path / "images/rootfs-1") > 0

    out, err, exitcode = run(
        f"rauc install http://127.0.0.1/test/good-verity-bundle.raucb --require-manifest-hash={BAD_HASH}"
    )
    assert exitcode == 1
    assert os.path.getsize(tmp_path / "images/rootfs-1") == 0

    out, err, exitcode = run(
        f"rauc install http://127.0.0.1/test/good-verity-bundle.raucb --require-manifest-hash={GOOD_HASH}"
    )
    assert exitcode == 0
    assert os.path.getsize(tmp_path / "images/rootfs-1") > 0



#!/usr/bin/env python3

import inspect
from datetime import datetime

from pyasn1.codec.der import decoder
from pyasn1.type import univ, char
from pyasn1_modules import rfc2315, rfc5280, rfc5480, rfc5652, rfc5751


def build_oid_map(modules):
    oid_map = {}
    for module in modules:
        for name, obj in inspect.getmembers(module):
            if isinstance(obj, univ.ObjectIdentifier):
                oid_str = str(obj)
                name = name.removeprefix("id_")
                oid_map[oid_str] = name
    return oid_map


OID_MAP = build_oid_map([rfc2315, rfc5280, rfc5480, rfc5652, rfc5751])


def resolve_oid(oid):
    return OID_MAP.get(str(oid), oid)


def decode_as_string(value_any):
    for asn1_type in [char.PrintableString, char.UTF8String]:
        try:
            decoded, _ = decoder.decode(value_any.asOctets(), asn1Spec=asn1_type())
            return str(decoded)
        except Exception:
            continue
    # fallback: return hex
    return value_any.prettyPrint()


def name_to_string(name):
    parts = []
    seq = name.getComponentByName("rdnSequence")
    for rdn in seq:  # each rdn is a SET OF AttributeTypeAndValue
        if isinstance(rdn, str):
            continue
        for attr in rdn:
            type_oid = attr.getComponentByName("type")
            value = attr.getComponentByName("value")
            value_str = decode_as_string(value)

            oid_map = {
                "2.5.4.3": "CN",
                "2.5.4.10": "O",
            }

            oid_str = str(type_oid)
            name = oid_map.get(oid_str, oid_str)

            parts.append(f"{name}={value_str}")

    return ", ".join(parts)


def utctime_to_datetime(utctime):
    """
    Decode an ASN.1 UTCTime string in YYMMDDHHMMSSZ format to a datetime object.
    Assumes:
      - YY in [00,49] => 2000..2049
      - YY in [50,99] => 1950..1999 (ASN.1 UTCTime rule)
    """

    assert isinstance(utctime, str)
    assert len(utctime) == 13
    assert utctime[-1] == "Z"

    year = int(utctime[0:2])
    if year < 50:
        year += 2000
    else:
        year += 1900
    month = int(utctime[2:4])
    day = int(utctime[4:6])
    hour = int(utctime[6:8])
    minute = int(utctime[8:10])
    second = int(utctime[10:12])

    return datetime(year, month, day, hour, minute, second)


def attribute_to_string(typ, attr):
    if typ == rfc5652.id_contentType:
        contentType, _ = decoder.decode(attr, asn1Spec=rfc5652.ContentType())
        return resolve_oid(str(contentType))
    elif typ == rfc5652.id_signingTime:
        signingTime, _ = decoder.decode(attr, asn1Spec=rfc5652.SigningTime())
        if signingTime.getName() == "utcTime":
            utcTime = str(signingTime.getComponent())
            return ("utcTime", utctime_to_datetime(utcTime))
        elif signingTime.getName() == "generalTime":
            return ("generalTime", str(signingTime.getComponent()))
        else:
            raise ValueError(f"unknown signingTime type {signingTime.getName()}")
    elif typ == rfc5652.id_messageDigest:
        messageDigest, _ = decoder.decode(attr, asn1Spec=rfc5652.MessageDigest())
        return bytes(messageDigest).hex()
    elif typ == rfc5751.smimeCapabilities:
        return ("smimeCapabilities", "(not decoded)")
    else:
        raise ValueError(f"unknown attribute type {resolve_oid(typ)}")


def decode_cms_signed_data(content_info):
    result = {}

    signed_data, _ = decoder.decode(content_info.getComponentByName("content"), asn1Spec=rfc5652.SignedData())

    result["digestAlgorithms"] = []
    for algo in signed_data.getComponentByName("digestAlgorithms"):
        result["digestAlgorithms"].append(resolve_oid(algo.getComponentByName("algorithm")))

    eci = signed_data.getComponentByName("encapContentInfo")
    content_type = eci.getComponentByName("eContentType")
    content = eci.getComponentByName("eContent")

    result["eContentType"] = resolve_oid(content_type)
    if content is not None and content.isValue:
        result["eContent"] = bytes(content)
    else:
        result["eContent"] = None

    certs = signed_data.getComponentByName("certificates")
    result["certs"] = []
    if certs is not None:
        for cert_choice in certs:
            if cert_choice.getName() == "certificate":
                cert = cert_choice.getComponent()
                tbs_cert = cert.getComponentByName("tbsCertificate")
                serial = tbs_cert.getComponentByName("serialNumber")
                issuer = tbs_cert.getComponentByName("issuer")
                subject = tbs_cert.getComponentByName("subject")
                result["certs"].append(
                    {
                        # all our certificates have serial numbers, so we can just convert here
                        "serial": int(serial),
                        "issuer": name_to_string(issuer),
                        "subject": name_to_string(subject),
                    }
                )

    signer_infos = signed_data.getComponentByName("signerInfos")
    result["signerInfos"] = []
    for signer_info in signer_infos:
        sid = signer_info.getComponentByName("sid")
        if sid.getName() == "issuerAndSerialNumber":
            iasn = sid.getComponent()  # get SEQUENCE inside the CHOICE
            issuer = iasn.getComponentByName("issuer")
            serial = iasn.getComponentByName("serialNumber")
        else:
            issuer = None
            serial = None
        result_attrs = {}
        attrs = signer_info.getComponentByName("signedAttrs")
        for attr in attrs:
            attr_type = attr.getComponentByName("attrType")
            attr_values = attr.getComponentByName("attrValues")
            assert len(attr_values) == 1
            result_attrs[resolve_oid(attr_type)] = attribute_to_string(attr_type, attr_values[0])
        digest_algorithm = signer_info.getComponentByName("digestAlgorithm")
        sig_algorithm = signer_info.getComponentByName("signatureAlgorithm")
        signature = signer_info.getComponentByName("signature")
        result["signerInfos"].append(
            {
                "issuer": name_to_string(issuer),
                "serial": int(serial),
                "digestAlgorithm": resolve_oid(digest_algorithm.getComponentByName("algorithm")),
                "signatureAlgorithm": resolve_oid(sig_algorithm.getComponentByName("algorithm")),
                "signedAttrs": result_attrs,
                "signature": bytes(signature),
            }
        )

    return result


def decode_cms_enveloped_data(content_info):
    result = {}

    enveloped_data, _ = decoder.decode(content_info.getComponentByName("content"), asn1Spec=rfc5652.EnvelopedData())

    eci = enveloped_data.getComponentByName("encryptedContentInfo")
    content_type = eci.getComponentByName("contentType")
    content = eci.getComponentByName("encryptedContent")

    result["contentType"] = resolve_oid(content_type)
    if content is not None and content.isValue:
        result["encryptedContent"] = bytes(content)
    else:
        result["encryptedContent"] = None

    return result


def decode_cms(der_data):
    content_info, _ = decoder.decode(der_data, asn1Spec=rfc5652.ContentInfo())

    content_type = content_info.getComponentByName("contentType")
    if content_type == rfc5652.id_signedData:
        result = decode_cms_signed_data(content_info)
    elif content_type == rfc5652.id_envelopedData:
        result = decode_cms_enveloped_data(content_info)
    else:
        raise ValueError("Not a supported CMS message.")

    return {
        "contentType": resolve_oid(content_type),
        **result,
    }


def main():
    import argparse
    import sys
    from pprint import pprint

    parser = argparse.ArgumentParser(
        description="Decode a CMS (Cryptographic Message Syntax) DER file and pretty-print its contents."
    )
    parser.add_argument("input_file", help="Path to the DER-encoded CMS file to decode.")

    args = parser.parse_args()

    try:
        with open(args.input_file, "rb") as f:
            der_data = f.read()
    except FileNotFoundError:
        print(f"Error: File not found: {args.input_file}", file=sys.stderr)
        sys.exit(1)
    except Exception as e:
        print(f"Error reading file {args.input_file}: {e}", file=sys.stderr)
        sys.exit(1)

    try:
        result = decode_cms(der_data)
        pprint(result, sort_dicts=False)
    except Exception as e:
        print(f"Error decoding CMS data: {e}", file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()
